/*
   A Davicom DM9000 ISA NIC fast Ethernet driver for Linux.
   Copyright (C) 1997  Sten Wang

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

  (C)Copyright 1997-1998 DAVICOM Semiconductor,Inc. All Rights Reserved.
*/
#include <windows.h>
#include <halether.h>
#include <oal.h>


#define DM9000_INFO		TRUE
#define DM9000_ERROR	TRUE
#define DM9000_DEBUG	FALSE
#define DM9000_MISS		TRUE

//------------------------------------------------------------------------------
// Types

enum DM9000_PHY_mode
{
	DM9000_10MHD = 0,
	DM9000_100MHD = 1,
	DM9000_10MFD = 4,
	DM9000_100MFD = 5,
	DM9000_AUTO = 8,
	DM9000_1M_HPNA = 0x10
};

#define DM9000_PHY_MODE	DM9000_AUTO

//------------------------------------------------------------------------------
// Defines 

#define CONFIG_DM9000_USE_32BIT

#define DM9000_ID		0x90000A46
#define DM9000_PKT_MAX		1536	/* Received packet max size */
#define DM9000_PKT_RDY		0x01	/* Packet ready to receive */

/* although the registers are 16 bit, they are 32-bit aligned.
 */

#define DM9000_NCR             0x00
#define DM9000_NSR             0x01
#define DM9000_TCR             0x02
#define DM9000_TSR1            0x03
#define DM9000_TSR2            0x04
#define DM9000_RCR             0x05
#define DM9000_RSR             0x06
#define DM9000_ROCR            0x07
#define DM9000_BPTR            0x08
#define DM9000_FCTR            0x09
#define DM9000_FCR             0x0A
#define DM9000_EPCR            0x0B
#define DM9000_EPAR            0x0C
#define DM9000_EPDRL           0x0D
#define DM9000_EPDRH           0x0E
#define DM9000_WCR             0x0F

#define DM9000_PAR             0x10
#define DM9000_MAR             0x16

#define DM9000_GPCR            0x1e
#define DM9000_GPR             0x1f
#define DM9000_TRPAL           0x22
#define DM9000_TRPAH           0x23
#define DM9000_RWPAL           0x24
#define DM9000_RWPAH           0x25

#define DM9000_VIDL            0x28
#define DM9000_VIDH            0x29
#define DM9000_PIDL            0x2A
#define DM9000_PIDH            0x2B

#define DM9000_CHIPR           0x2C
#define DM9000_SMCR            0x2F

#define DM9000_PHY             0x40

#define DM9000_MRCMDX          0xF0
#define DM9000_MRCMD           0xF2
#define DM9000_MRRL            0xF4
#define DM9000_MRRH            0xF5
#define DM9000_MWCMDX			0xF6
#define DM9000_MWCMD           0xF8
#define DM9000_MWRL            0xFA
#define DM9000_MWRH            0xFB
#define DM9000_TXPLL           0xFC
#define DM9000_TXPLH           0xFD
#define DM9000_ISR             0xFE
#define DM9000_IMR             0xFF

#define NCR_EXT_PHY		(1<<7)
#define NCR_WAKEEN		(1<<6)
#define NCR_FCOL		(1<<4)
#define NCR_FDX			(1<<3)
#define NCR_LBK			(3<<1)
#define NCR_RST			(1<<0)

#define NSR_SPEED		(1<<7)
#define NSR_LINKST		(1<<6)
#define NSR_WAKEST		(1<<5)
#define NSR_TX2END		(1<<3)
#define NSR_TX1END		(1<<2)
#define NSR_RXOV		(1<<1)

#define TCR_TJDIS		(1<<6)
#define TCR_EXCECM		(1<<5)
#define TCR_PAD_DIS2	(1<<4)
#define TCR_CRC_DIS2	(1<<3)
#define TCR_PAD_DIS1	(1<<2)
#define TCR_CRC_DIS1	(1<<1)
#define TCR_TXREQ		(1<<0)

#define TSR_TJTO		(1<<7)
#define TSR_LC			(1<<6)
#define TSR_NC			(1<<5)
#define TSR_LCOL		(1<<4)
#define TSR_COL			(1<<3)
#define TSR_EC			(1<<2)

#define RCR_WTDIS		(1<<6)
#define RCR_DIS_LONG	(1<<5)
#define RCR_DIS_CRC		(1<<4)
#define RCR_ALL			(1<<3)
#define RCR_RUNT		(1<<2)
#define RCR_PRMSC		(1<<1)
#define RCR_RXEN		(1<<0)

#define RSR_RF			(1<<7)
#define RSR_MF			(1<<6)
#define RSR_LCS			(1<<5)
#define RSR_RWTO		(1<<4)
#define RSR_PLE			(1<<3)
#define RSR_AE			(1<<2)
#define RSR_CE			(1<<1)
#define RSR_FOE			(1<<0)

#define FCTR_HWOT(ot)	(( ot & 0xf ) << 4 )
#define FCTR_LWOT(ot)	( ot & 0xf )

#define IMR_PAR			(1<<7)
#define IMR_ROOM		(1<<3)
#define IMR_ROM			(1<<2)
#define IMR_PTM			(1<<1)
#define IMR_PRM			(1<<0)

//------------------------------------------------------------------------------
// Local Variables 

static UINT32	DM9000_IO;
static UINT32	DM9000_DATA;

//------------------------------------------------------------------------------
// Local Functions 
typedef UINT32	u32;
typedef UINT16	u16;
typedef UINT8	u8;

#define DM9000_outb(d,r) ( *(volatile u8 *)r = d )
#define DM9000_outw(d,r) ( *(volatile u16 *)r = d )
#define DM9000_outl(d,r) ( *(volatile u32 *)r = d )
#define DM9000_inb(r) (*(volatile u8 *)r)
#define DM9000_inw(r) (*(volatile u16 *)r)
#define DM9000_inl(r) (*(volatile u32 *)r)

static u8 DM9000_ior(int reg)
{
	DM9000_outb(reg, DM9000_IO);
	return DM9000_inb(DM9000_DATA);
}

static void DM9000_iow(int reg, u8 value)
{
	DM9000_outb(reg, DM9000_IO);
	DM9000_outb(value, DM9000_DATA);
}

static u16 dm9000_phy_read(int reg)
{
   u16 val;

   /* Fill the phyxcer register into REG_0C */
   DM9000_iow(DM9000_EPAR, DM9000_PHY | reg);
   DM9000_iow(DM9000_EPCR, 0xc);   /* Issue phyxcer read command */
   OALStall(100);      /* Wait read complete */
   DM9000_iow(DM9000_EPCR, 0x0);   /* Clear phyxcer read command */
   val = (DM9000_ior(DM9000_EPDRH) << 8) | DM9000_ior(DM9000_EPDRL);

   /* The read data keeps on REG_0D & REG_0E */
//   DM9000_DBG("phy_read(%d): %d\n", reg, val);
   return val;
}

static void dm9000_phy_write(int reg, u16 value)
{

   /* Fill the phyxcer register into REG_0C */
   DM9000_iow(DM9000_EPAR, DM9000_PHY | reg);

   /* Fill the written data into REG_0D & REG_0E */
   DM9000_iow(DM9000_EPDRL, (value & 0xff));
   DM9000_iow(DM9000_EPDRH, ((value >> 8) & 0xff));
   DM9000_iow(DM9000_EPCR, 0xa);   /* Issue phyxcer write command */
   OALStall(500);      /* Wait write complete */
   DM9000_iow(DM9000_EPCR, 0x0);   /* Clear phyxcer write command */
//   DM9000_DBG("phy_write(reg:%d, value:%d)\n", reg, value);
}

//------------------------------------------------------------------------------

static u8 dm9000_io_width[4] = { 16, 32, 8, 0 };

static BOOL dm9000_probe(void)
{
	u32 i, id;
	u8 iowd = 0;

	for( i=0; i<3; i++)
	{
		id = DM9000_ior(DM9000_VIDL);
		id |= DM9000_ior(DM9000_VIDH) << 8;
		id |= DM9000_ior(DM9000_PIDL) << 16;
		id |= DM9000_ior(DM9000_PIDH) << 24;

		if (id == DM9000_ID)
		{
			iowd = (DM9000_ior(DM9000_ISR) & 0xC0) >> 6;

			OALMSGS(DM9000_INFO, (L"DM9000 I/O: 0x%08x (%dbit), id: 0x%08x\r\n",
									DM9000_IO, dm9000_io_width[iowd], id));
			return TRUE;
		} else {
			OALMSGS(DM9000_INFO, (L"DM9000 not found at I/O: 0x%08x, id: 0x%08x\r\n", DM9000_IO, id));
		}
	}
	return FALSE;
}

static void dm9000_set_mode(u8 phy_mode)
{
	u16 phy_reg4 = 0x01e1, phy_reg0 = 0x1000;

	if (!(phy_mode & DM9000_AUTO)) {
		switch (phy_mode) {
		case DM9000_10MHD:
			phy_reg4 = 0x21;
			phy_reg0 = 0x0000;
			break;
		case DM9000_10MFD:
			phy_reg4 = 0x41;
			phy_reg0 = 0x1100;
			break;
		case DM9000_100MHD:
			phy_reg4 = 0x81;
			phy_reg0 = 0x2000;
			break;
		case DM9000_100MFD:
			phy_reg4 = 0x101;
			phy_reg0 = 0x3100;
			break;
		}
		dm9000_phy_write(4, phy_reg4);
		dm9000_phy_write(0, phy_reg0);
	}
	DM9000_iow(DM9000_GPCR, 0x01);   /* Let GPIO0 output */
	DM9000_iow(DM9000_GPR, 0x00);   /* Enable PHY */
}

#ifdef CONFIG_DM9000_SROM
static u16 dm9000_read_srom(int offset)
{
   DM9000_iow(DM9000_EPAR, offset);
   DM9000_iow(DM9000_EPCR, 0x4);
   OALStall(200);
   DM9000_iow(DM9000_EPCR, 0x0);
   return (DM9000_ior(DM9000_EPDRL) + (DM9000_ior(DM9000_EPDRH) << 8));
}
#endif

static void dm9000_reset(void)
{
    OALMSGS(DM9000_DEBUG, (L"dm9000_reset\r\n"));
	
	DM9000_iow(DM9000_NCR, NCR_RST);
	OALStall(1000);      /* delay 1ms */
}

BOOL DM9000Init(UINT8 *pAddress, UINT32 offset, UINT16 mac[3])
{
	int i, oft, lnk;
	UINT8* pMAC = (UINT8*) mac;

	DM9000_IO = (UINT32) pAddress;
	DM9000_DATA = DM9000_IO + 4;

	OALMSGS(DM9000_DEBUG, (L"+DM9000Init I/O: 0x%08x\r\n", DM9000_IO));

	/* RESET device */
	dm9000_reset();
	dm9000_probe();

	/* NIC Type: FASTETHER, HOMERUN, LONGRUN */
//	identify_nic();

	/* GPIO0 on pre-activate PHY */
	DM9000_iow(DM9000_GPR, 0x00);   /*REG_1F bit0 activate phyxcer */

	/* Set PHY */
	dm9000_set_mode(DM9000_PHY_MODE);

	/* Program operating register */
	DM9000_iow(DM9000_NCR, 0x0);   /* only intern phy supported by now */
	DM9000_iow(DM9000_TCR, 0);   /* TX Polling clear */
//	DM9000_iow(DM9000_BPTR, 0x3f);   /* Less 3Kb, 200us */
//   DM9000_iow(DM9000_FCTR, FCTR_HWOT(3) | FCTR_LWOT(8));   /* Flow Control : High/Low Water */
//   DM9000_iow(DM9000_FCR, 0xff);   /* enable Flow Control */
	DM9000_iow(DM9000_SMCR, 0);   /* Special Mode */
	DM9000_iow(DM9000_NSR, NSR_WAKEST | NSR_TX2END | NSR_TX1END);   /* clear TX status */
	DM9000_iow(DM9000_ISR, 0x0f);   /* Clear interrupt status */

	/* Set Node address */
	for (i = 0; i < 6; i++)
#ifdef CONFIG_DM9000_SROM
		pMAC[i] = dm9000_read_srom(i);
#else
		pMAC[i] = i;
#endif
	OALMSGS(DM9000_INFO, (L"DM9000 MAC: %02x:%02x:%02x:%02x:%02x:%02x\r\n",
				pMAC[0], pMAC[1], pMAC[2], pMAC[3], pMAC[4], pMAC[5]));
	for (i = 0, oft = 0x10; i < 6; i++, oft++)
		DM9000_iow(oft, pMAC[i]);
	for (i = 0, oft = 0x16; i < 8; i++, oft++)
		DM9000_iow(oft, 0xff);

	/* read back mac, just to be sure */
//	for (i = 0, oft = 0x10; i < 6; i++, oft++)
//		DM9000_DBG("%02x:", DM9000_ior(oft));
//	DM9000_DBG("\n");

	/* Activate DM9000 */
	DM9000_iow(DM9000_RCR, RCR_DIS_LONG | RCR_DIS_CRC | RCR_RXEN);   /* RX enable */
    DM9000_iow(DM9000_IMR, IMR_PAR | IMR_PTM | IMR_PRM);   /* Enable TX/RX interrupt mask */
//	DM9000_iow(DM9000_IMR, IMR_PAR);

	for( i=0; !(dm9000_phy_read(1) & 0x20); i++) {   /* autonegation complete bit */
		OALStall(1000);
		if (i == 10000) {
			OALMSGS(DM9000_ERROR, (L"DM9000 could not establish link\r\n"));
			return FALSE;
		}
	}

	/* see what we've got */
	lnk = dm9000_phy_read(17) >> 12;
	OALMSGS(DM9000_INFO, (L"DM9000 operating at %d mode\r\n", lnk));

	OALMSGS(DM9000_DEBUG, (L"-DM9000Init\r\n"));
	return TRUE;
}


//------------------------------------------------------------------------------

UINT16 DM9000SendFrame(UINT8 *pBuffer, UINT32 length)
{
	u8 *data_ptr;
	u32 tmplen, i;
//	UINT32 startTime;

	/* wait for end of transmission */
    // Wait until it is sent or an error is generated.
	while( DM9000_ior(DM9000_TCR) & TCR_TXREQ);

	OALMSGS(DM9000_DEBUG, (L"+DM9000SendFrame %d bytes\r\n", length));
#ifdef CONFIG_DM9000_DEBUG
	for (i = 0; i < length; i++) {
		if (i % 8 == 0)
			DM9000_DBG("\nSend: %02x: ", (unsigned int)i);
		DM9000_DBG("%02x ", ((unsigned char *) packet)[i]);
	}
	DM9000_DBG("\n");
#endif
	/* Move data to DM9000 TX RAM */
	data_ptr = (u8 *) pBuffer;
	DM9000_outb(DM9000_MWCMD, DM9000_IO);

#ifdef CONFIG_DM9000_USE_8BIT
	/* Byte mode */
	for (i = 0; i < length; i++)
		DM9000_outb((data_ptr[i] & 0xff), DM9000_DATA);
#endif            /*  */
#ifdef CONFIG_DM9000_USE_16BIT
	tmplen = (length + 1) / 2;
	for (i = 0; i < tmplen; i++)
		DM9000_outw(((u16 *) data_ptr)[i], DM9000_DATA);
#endif            /*  */
#ifdef CONFIG_DM9000_USE_32BIT
	tmplen = (length + 3) / 4;
	for (i = 0; i < tmplen; i++)
		DM9000_outl(((u32 *) data_ptr)[i], DM9000_DATA);
#endif            /*  */

	/* Set TX length to DM9000 */
	DM9000_iow(DM9000_TXPLL, length & 0xff);
	DM9000_iow(DM9000_TXPLH, (length >> 8) & 0xff);

	/* Issue TX command */
	DM9000_iow(DM9000_TCR, TCR_TXREQ);   /* Cleared after TX complete */

    OALMSGS(DM9000_DEBUG, (L"-DM9000SendFrame\r\n"));
	return 0;
}

//------------------------------------------------------------------------------

UINT16 DM9000GetFrame(UINT8 *pBuffer, UINT16 *pLength)
{
   u8 rxbyte, *rdptr = (u8 *) pBuffer;
   u16 IntStatus, RxStatus, RxLen = 0;
   u32 tmplen, i;

    OALMSGS(DM9000_DEBUG, (L"+DM9000GetFrame %d bytes\r\n", *pLength));

	*pLength = 0;

	IntStatus = DM9000_ior(DM9000_ISR);       /* Got ISR */
	DM9000_iow(DM9000_ISR, (u8)IntStatus);        /* Clear ISR status */
   
	/* Check packet ready or not */
	DM9000_ior(DM9000_MRCMDX);   /* Dummy read */
	rxbyte = DM9000_inb(DM9000_DATA);   /* Got most updated data */
	if (rxbyte == 0)
		return 0;

	/* Status check: this byte must be 0 or 1 */
	if (rxbyte > 1) {
		DM9000_iow(DM9000_RCR, 0x00);   /* Stop Device */
//		DM9000_iow(DM9000_ISR, 0x80);   /* Stop INT request */
	    OALMSGS(DM9000_ERROR, (L"DM9000 RX status check FAILED (0x%02x)\r\n", rxbyte));
	}

	/* A packet ready now  & Get status/length */
	DM9000_outb(DM9000_MRCMD, DM9000_IO);

#ifdef CONFIG_DM9000_USE_8BIT
	RxStatus = DM9000_inb(DM9000_DATA) + (DM9000_inb(DM9000_DATA) << 8);
	RxLen = DM9000_inb(DM9000_DATA) + (DM9000_inb(DM9000_DATA) << 8);
#endif            /*  */
#ifdef CONFIG_DM9000_USE_16BIT
	RxStatus = DM9000_inw(DM9000_DATA);
	RxLen = DM9000_inw(DM9000_DATA);
#endif            /*  */
#ifdef CONFIG_DM9000_USE_32BIT
	tmplen = DM9000_inl(DM9000_DATA);
	RxStatus = tmplen;
	RxLen = tmplen >> 16;
#endif            /*  */
	OALMSGS(DM9000_DEBUG, (L"DM9000 RX status: 0x%04x, RX len: %d\r\n", RxStatus, RxLen));

	/* Move data from DM9000 */
	/* Read received packet from RX SRAM */
#ifdef CONFIG_DM9000_USE_8BIT
	for (i = 0; i < RxLen; i++)
		rdptr[i] = DM9000_inb(DM9000_DATA);
#endif            /*  */
#ifdef CONFIG_DM9000_USE_16BIT
	tmplen = (RxLen + 1) / 2;
	for (i = 0; i < tmplen; i++)
		((u16 *) rdptr)[i] = DM9000_inw(DM9000_DATA);
#endif            /*  */
#ifdef CONFIG_DM9000_USE_32BIT
	tmplen = (RxLen + 3) / 4;
	for (i = 0; i < tmplen; i++)
		((u32 *) rdptr)[i] = DM9000_inl(DM9000_DATA);
#endif            /*  */
	if ((RxStatus & 0xbf00) || (RxLen < 0x40)
				|| (RxLen > DM9000_PKT_MAX)) {
		OALMSGS(DM9000_ERROR, (L"DM9000 RX error (0x%04x)\r\n", RxStatus));
//		dm9000_reset();
	} else {
#ifdef CONFIG_DM9000_DEBUG
		for (i = 0; i < RxLen; i++) {
			if (i % 8 == 0)
				DM9000_DBG("\nRecv: %02x: ", (unsigned int)i);
			DM9000_DBG("%02x ", rdptr[i]);
		} 
		DM9000_DBG("\n");
#endif
		*pLength = RxLen;
	}
	OALMSGS(DM9000_DEBUG, (L"-DM9000GetFrame\r\n"));

	return *pLength;
}


//------------------------------------------------------------------------------

VOID DM9000EnableInts()
{
    OALMSGS(DM9000_DEBUG, (L"+DM9000EnableInts\r\n"));

    DM9000_iow(DM9000_IMR, IMR_PAR | IMR_PTM | IMR_PRM);   /* Enable TX/RX interrupt mask */

    OALMSGS(DM9000_DEBUG, (L"-DM9000EnableInts\r\n"));
}

//------------------------------------------------------------------------------

VOID DM9000DisableInts()
{
    OALMSGS(DM9000_DEBUG, (L"+DM9000DisableInts\r\n"));

	DM9000_iow(DM9000_IMR, IMR_PAR);   /* Disable TX/RX interrupt mask */
	DM9000_iow(DM9000_ISR, 0xf);
    
	OALMSGS(DM9000_DEBUG, (L"-DM9000DisableInts\r\n"));
}

//------------------------------------------------------------------------------

VOID DM9000CurrentPacketFilter(UINT32 filter)
{
    OALMSGS(DM9000_MISS, (L"+DM9000CurrentPacketFilter(0x%08x)\r\n", filter));
    OALMSGS(DM9000_MISS, (L"-DM9000CurrentPacketFilter\r\n"));
}

//------------------------------------------------------------------------------

BOOL DM9000MulticastList(UINT8 *pAddresses, UINT32 count)
{
    OALMSGS(DM9000_MISS, (L"+DM9000MulticastList(0x%08x, %d)\r\n", pAddresses, count));
    OALMSGS(DM9000_MISS, (L"-DM9000MulticastList\r\n"));
    return TRUE;
}

//------------------------------------------------------------------------------
