//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
//
/****************************************************************************** 
** Copyright 2000-2003 Intel Corporation All Rights Reserved.
**
** Portions of the source code contained or described herein and all documents
** related to such source code (Material) are owned by Intel Corporation
** or its suppliers or licensors and is licensed by Microsoft Corporation for distribution.  
** Title to the Material remains with Intel Corporation or its suppliers and licensors. 
** Use of the Materials is subject to the terms of the Microsoft license agreement which accompanied the Materials.  
** No other license under any patent, copyright, trade secret or other intellectual
** property right is granted to or conferred upon you by disclosure or
** delivery of the Materials, either expressly, by implication, inducement,
** estoppel or otherwise 
** Some portion of the Materials may be copyrighted by Microsoft Corporation.
**
********************************************************************************/
#include <windows.h>
#include <pm.h>

#include <types.h>
#include <string.h>
#include <stdio.h>
#include <tchar.h>

#include <ceddk.h>
#include <bulverde.h>
#include <mainstoneii.h>
#include <xllp_i2c.h>


#include "Cs.h"
#include "Csmedia.h"

#include "CameraPDDProps.h"
#include "dstruct.h"
#include "dbgsettings.h"
#include <camera.h>
#include "CameraDriver.h"
#include "SensorFormats.h"
#include "SensorProperties.h"
#include "PinDriver.h"
extern "C" {
#include "bulverde_camera.h"
}
#include "cameraPdd.h"
#include "wchar.h"
#include "pdd_intf.h"


extern "C" {
#include "ADCM2650.h"
}

#define WIDTHBYTES(bits) ((DWORD)(((bits)+31) & (~31)) / 8)

void CameraVideoFrameCallback(DWORD dwContext);

void CameraStillFrameCallback(DWORD dwContext);


PDDFUNCTBL FuncTbl = {
    sizeof(PDDFUNCTBL),
    PDD_Init,
    PDD_DeInit,
    PDD_GetAdapterInfo,
    PDD_HandleVidProcAmpChanges,
    PDD_HandleCamControlChanges,
    PDD_HandleVideoControlCapsChanges,
    PDD_SetPowerState,
    PDD_HandleAdapterCustomProperties,
    PDD_InitSensorMode,
    PDD_DeInitSensorMode,
    PDD_SetSensorState,
    PDD_TakeStillPicture,
    PDD_GetSensorModeInfo,
    PDD_SetSensorModeFormat,
    PDD_AllocateBuffer,
    PDD_DeAllocateBuffer,
    PDD_RegisterClientBuffer,
    PDD_UnRegisterClientBuffer,
    PDD_FillBuffer,
    PDD_HandleModeCustomProperties
};

const POWER_CAPABILITIES s_PowerCaps = 
{
    // DeviceDx:    Supported power states
    DX_MASK(D0 )| DX_MASK(D2) | DX_MASK(D3) | DX_MASK(D4),

    0,              // WakeFromDx:
    0,              // InrushDx:    No inrush of power

    {               // Power: Maximum milliwatts in each state
        0x00000001, //        D0 = 0
        0x00000001, //        D1 = 0
        0x00000001, //        D2 = 0
        0x00000001, //        D3 = 0
        0x00000001  //        D4 = 0 (off)
    },

    {               // Latency
        0x00000000, //        D0 = 0
        0x00000000, //        D1 = 0
        0x00000000, //        D2 = 0
        0x00000000, //        D3 = 0
        0x00000000  //        D4 = 0
    },

    0,                    // Flags: None
};


CCameraPdd::CCameraPdd()
{
    m_ulCTypes = 2;
    m_hContext = NULL;
    m_pModeVideoFormat = NULL;
    m_pModeVideoCaps = NULL;
    m_ppModeContext = NULL;
    m_ulCurrentFrame = 0;
    m_bCameraHWRunning = FALSE;
    m_bStillInProgress = FALSE;
    m_PowerState = D0;

    
    memset( &m_CsState, 0x0, sizeof(m_CsState));
    memset( &m_CsPrevState, 0x0, sizeof(m_CsPrevState));
    memset( &m_SensorModeInfo, 0x0, sizeof(m_SensorModeInfo));
    memset( &m_SensorProps, 0x0, sizeof(m_SensorProps));
    memset( &PowerCaps, 0x0, sizeof(PowerCaps));
    memset( &m_CurrentFormat, 0x0, sizeof(m_CurrentFormat));

}

CCameraPdd::~CCameraPdd()
{
    if( NULL != m_pModeVideoCaps )
    {
        delete [] m_pModeVideoCaps;
        m_pModeVideoCaps = NULL;
    }

    if( NULL != m_pModeVideoFormat )
    {
        delete [] m_pModeVideoFormat;
        m_pModeVideoFormat = NULL;
    }

    if( NULL != m_ppModeContext )
    {
        delete [] m_ppModeContext;
        m_ppModeContext = NULL;
    }
}

DWORD CCameraPdd::PDDInit( PVOID MDDContext, PPDDFUNCTBL pPDDFuncTbl )
{
    DWORD hr = ERROR_SUCCESS;

    if(!MDDContext)
    {
        DEBUGMSG( ZONE_ERROR, ( _T("PddInit:FAILED NULL MDD context\r\n") ) );
        hr = ERROR_INVALID_PARAMETER;
        goto ErrorExit;
    }
    // Save the MDD context for future use...
    m_hContext = (HANDLE)MDDContext;

    // Call CameraInit() 
    // It initializes Bulverde registers, create events and interrupt thread,
    // and initialize sensor with default parameter.
    if (!CameraInit(MDDContext))
    {
        DEBUGMSG( ZONE_ERROR, ( _T("PddInit:FAILED to Initialize camera\r\n") ) );
        CameraDeinit();
        goto ErrorExit;
    }

    dwCameraDriverContext = reinterpret_cast<DWORD>( this );
    // This callback function is used to signal the application that the video frame is now available.
    pfnCameraHandleVideoFrame = CameraVideoFrameCallback;
    // This callback function is used to signal the application that the still image is now available.
    pfnCameraHandleStillFrame = CameraStillFrameCallback;


    // We support only 2 PIN driver. Preview and Still.
    m_ulCTypes = 2; // Default number of Sensor Modes is 2
    
    // Return the function table if you have sufficient memory.
    if( pPDDFuncTbl->dwSize  > sizeof( PDDFUNCTBL ) )
    {
        DEBUGMSG( ZONE_ERROR, ( _T("PddInit:Out of Memory\r\n") ) );
        hr = ERROR_INSUFFICIENT_BUFFER;
        goto ErrorExit;
    }

    memcpy( pPDDFuncTbl, &FuncTbl, sizeof( PDDFUNCTBL ) );

    memset( m_SensorProps, 0x0, sizeof(m_SensorProps) );

    memcpy( &PowerCaps, &s_PowerCaps, sizeof( POWER_CAPABILITIES ) );
    

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Set all VideoProcAmp and CameraControl properties.
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //VideoProcAmp
    m_SensorProps[ENUM_BRIGHTNESS].ulCurrentValue     = BrightnessDefault;
    m_SensorProps[ENUM_BRIGHTNESS].ulDefaultValue     = BrightnessDefault;
    m_SensorProps[ENUM_BRIGHTNESS].pRangeNStep        = &BrightnessRangeAndStep[0];
    m_SensorProps[ENUM_BRIGHTNESS].ulFlags            = CSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
    m_SensorProps[ENUM_BRIGHTNESS].ulCapabilities     = CSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL|CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_BRIGHTNESS].fSetSupported      = VideoProcAmpProperties[ENUM_BRIGHTNESS].SetSupported;
    m_SensorProps[ENUM_BRIGHTNESS].fGetSupported      = VideoProcAmpProperties[ENUM_BRIGHTNESS].GetSupported;
    m_SensorProps[ENUM_BRIGHTNESS].pCsPropValues      = &BrightnessValues;

    m_SensorProps[ENUM_CONTRAST].ulCurrentValue       = ContrastDefault;
    m_SensorProps[ENUM_CONTRAST].ulDefaultValue       = ContrastDefault;
    m_SensorProps[ENUM_CONTRAST].pRangeNStep          = &ContrastRangeAndStep[0];
    m_SensorProps[ENUM_CONTRAST].ulFlags              = CSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL;
    m_SensorProps[ENUM_CONTRAST].ulCapabilities       = CSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL|CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_CONTRAST].fSetSupported        = VideoProcAmpProperties[ENUM_CONTRAST].SetSupported;
    m_SensorProps[ENUM_CONTRAST].fGetSupported        = VideoProcAmpProperties[ENUM_CONTRAST].GetSupported;
    m_SensorProps[ENUM_CONTRAST].pCsPropValues        = &ContrastValues;

    m_SensorProps[ENUM_HUE].ulCurrentValue            = HueDefault;
    m_SensorProps[ENUM_HUE].ulDefaultValue            = HueDefault;
    m_SensorProps[ENUM_HUE].pRangeNStep               = &HueRangeAndStep[0];
    m_SensorProps[ENUM_HUE].ulFlags                   = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_HUE].ulCapabilities            = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_HUE].fSetSupported             = VideoProcAmpProperties[ENUM_HUE].SetSupported;
    m_SensorProps[ENUM_HUE].fGetSupported             = VideoProcAmpProperties[ENUM_HUE].GetSupported;
    m_SensorProps[ENUM_HUE].pCsPropValues             = &HueValues;

    m_SensorProps[ENUM_SATURATION].ulCurrentValue     = SaturationDefault;
    m_SensorProps[ENUM_SATURATION].ulDefaultValue     = SaturationDefault;
    m_SensorProps[ENUM_SATURATION].pRangeNStep        = &SaturationRangeAndStep[0];
    m_SensorProps[ENUM_SATURATION].ulFlags            = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_SATURATION].ulCapabilities     = CSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL|CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_SATURATION].fSetSupported      = VideoProcAmpProperties[ENUM_SATURATION].SetSupported;
    m_SensorProps[ENUM_SATURATION].fGetSupported      = VideoProcAmpProperties[ENUM_SATURATION].GetSupported;
    m_SensorProps[ENUM_SATURATION].pCsPropValues      = &SaturationValues;

    m_SensorProps[ENUM_SHARPNESS].ulCurrentValue      = SharpnessDefault;
    m_SensorProps[ENUM_SHARPNESS].ulDefaultValue      = SharpnessDefault;
    m_SensorProps[ENUM_SHARPNESS].pRangeNStep         = &SharpnessRangeAndStep[0];
    m_SensorProps[ENUM_SHARPNESS].ulFlags             = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_SHARPNESS].ulCapabilities      = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_SHARPNESS].fSetSupported       = VideoProcAmpProperties[ENUM_SHARPNESS].SetSupported;
    m_SensorProps[ENUM_SHARPNESS].fGetSupported       = VideoProcAmpProperties[ENUM_SHARPNESS].GetSupported;
    m_SensorProps[ENUM_SHARPNESS].pCsPropValues       = &SharpnessValues;

    m_SensorProps[ENUM_GAMMA].ulCurrentValue          = GammaDefault;
    m_SensorProps[ENUM_GAMMA].ulDefaultValue          = GammaDefault;
    m_SensorProps[ENUM_GAMMA].pRangeNStep             = &GammaRangeAndStep[0];
    m_SensorProps[ENUM_GAMMA].ulFlags                 = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_GAMMA].ulCapabilities          = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_GAMMA].fSetSupported           = VideoProcAmpProperties[ENUM_GAMMA].SetSupported;
    m_SensorProps[ENUM_GAMMA].fGetSupported           = VideoProcAmpProperties[ENUM_GAMMA].GetSupported;
    m_SensorProps[ENUM_GAMMA].pCsPropValues           = &GammaValues;

    m_SensorProps[ENUM_COLORENABLE].ulCurrentValue    = ColorEnableDefault;
    m_SensorProps[ENUM_COLORENABLE].ulDefaultValue    = ColorEnableDefault;
    m_SensorProps[ENUM_COLORENABLE].pRangeNStep       = &ColorEnableRangeAndStep[0];
    m_SensorProps[ENUM_COLORENABLE].ulFlags           = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_COLORENABLE].ulCapabilities    = CSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL|CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_COLORENABLE].fSetSupported     = VideoProcAmpProperties[ENUM_COLORENABLE].SetSupported;
    m_SensorProps[ENUM_COLORENABLE].fGetSupported     = VideoProcAmpProperties[ENUM_COLORENABLE].GetSupported;
    m_SensorProps[ENUM_COLORENABLE].pCsPropValues     = &ColorEnableValues;

    m_SensorProps[ENUM_WHITEBALANCE].ulCurrentValue   = WhiteBalanceDefault;
    m_SensorProps[ENUM_WHITEBALANCE].ulDefaultValue   = WhiteBalanceDefault;
    m_SensorProps[ENUM_WHITEBALANCE].pRangeNStep      = &WhiteBalanceRangeAndStep[0];
    m_SensorProps[ENUM_WHITEBALANCE].ulFlags          = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_WHITEBALANCE].ulCapabilities   = CSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL|CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_WHITEBALANCE].fSetSupported    = VideoProcAmpProperties[ENUM_WHITEBALANCE].SetSupported;
    m_SensorProps[ENUM_WHITEBALANCE].fGetSupported    = VideoProcAmpProperties[ENUM_WHITEBALANCE].GetSupported;
    m_SensorProps[ENUM_WHITEBALANCE].pCsPropValues    = &WhiteBalanceValues;

    m_SensorProps[ENUM_BACKLIGHT_COMPENSATION].ulCurrentValue = BackLightCompensationDefault;
    m_SensorProps[ENUM_BACKLIGHT_COMPENSATION].ulDefaultValue = BackLightCompensationDefault;
    m_SensorProps[ENUM_BACKLIGHT_COMPENSATION].pRangeNStep    = &BackLightCompensationRangeAndStep[0];
    m_SensorProps[ENUM_BACKLIGHT_COMPENSATION].ulFlags        = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_BACKLIGHT_COMPENSATION].ulCapabilities = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_BACKLIGHT_COMPENSATION].fSetSupported  = VideoProcAmpProperties[ENUM_BACKLIGHT_COMPENSATION].SetSupported;
    m_SensorProps[ENUM_BACKLIGHT_COMPENSATION].fGetSupported  = VideoProcAmpProperties[ENUM_BACKLIGHT_COMPENSATION].GetSupported;
    m_SensorProps[ENUM_BACKLIGHT_COMPENSATION].pCsPropValues  = &BackLightCompensationValues;

    m_SensorProps[ENUM_GAIN].ulCurrentValue           = GainDefault;
    m_SensorProps[ENUM_GAIN].ulDefaultValue           = GainDefault;
    m_SensorProps[ENUM_GAIN].pRangeNStep              = &GainRangeAndStep[0];
    m_SensorProps[ENUM_GAIN].ulFlags                  = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_GAIN].ulCapabilities           = CSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;
    m_SensorProps[ENUM_GAIN].fSetSupported            = VideoProcAmpProperties[ENUM_GAIN].SetSupported;
    m_SensorProps[ENUM_GAIN].fGetSupported            = VideoProcAmpProperties[ENUM_GAIN].GetSupported;
    m_SensorProps[ENUM_GAIN].pCsPropValues            = &GainValues;

    //CameraControl
    m_SensorProps[ENUM_PAN].ulCurrentValue            = PanDefault;
    m_SensorProps[ENUM_PAN].ulDefaultValue            = PanDefault;
    m_SensorProps[ENUM_PAN].pRangeNStep               = &PanRangeAndStep[0];
    m_SensorProps[ENUM_PAN].ulFlags                   = CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_PAN].ulCapabilities            = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL|CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_PAN].fSetSupported             = VideoProcAmpProperties[ENUM_PAN-NUM_VIDEOPROCAMP_ITEMS].SetSupported;
    m_SensorProps[ENUM_PAN].fGetSupported             = VideoProcAmpProperties[ENUM_PAN-NUM_VIDEOPROCAMP_ITEMS].GetSupported;
    m_SensorProps[ENUM_PAN].pCsPropValues             = &PanValues;

    m_SensorProps[ENUM_TILT].ulCurrentValue           = TiltDefault;
    m_SensorProps[ENUM_TILT].ulDefaultValue           = TiltDefault;
    m_SensorProps[ENUM_TILT].pRangeNStep              = &TiltRangeAndStep[0];
    m_SensorProps[ENUM_TILT].ulFlags                  = CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_TILT].ulCapabilities           = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL|CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_TILT].fSetSupported            = VideoProcAmpProperties[ENUM_TILT-NUM_VIDEOPROCAMP_ITEMS].SetSupported;
    m_SensorProps[ENUM_TILT].fGetSupported            = VideoProcAmpProperties[ENUM_TILT-NUM_VIDEOPROCAMP_ITEMS].GetSupported;
    m_SensorProps[ENUM_TILT].pCsPropValues            = &TiltValues;

    m_SensorProps[ENUM_ROLL].ulCurrentValue           = RollDefault;
    m_SensorProps[ENUM_ROLL].ulDefaultValue           = RollDefault;
    m_SensorProps[ENUM_ROLL].pRangeNStep              = &RollRangeAndStep[0];
    m_SensorProps[ENUM_ROLL].ulFlags                  = CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_ROLL].ulCapabilities           = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL|CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_ROLL].fSetSupported            = VideoProcAmpProperties[ENUM_ROLL-NUM_VIDEOPROCAMP_ITEMS].SetSupported;
    m_SensorProps[ENUM_ROLL].fGetSupported            = VideoProcAmpProperties[ENUM_ROLL-NUM_VIDEOPROCAMP_ITEMS].GetSupported;
    m_SensorProps[ENUM_ROLL].pCsPropValues            = &RollValues;

    m_SensorProps[ENUM_ZOOM].ulCurrentValue           = ZoomDefault;
    m_SensorProps[ENUM_ZOOM].ulDefaultValue           = ZoomDefault;
    m_SensorProps[ENUM_ZOOM].pRangeNStep              = &ZoomRangeAndStep[0];
    m_SensorProps[ENUM_ZOOM].ulFlags                  = CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_ZOOM].ulCapabilities           = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL|CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_ZOOM].fSetSupported            = VideoProcAmpProperties[ENUM_ZOOM-NUM_VIDEOPROCAMP_ITEMS].SetSupported;
    m_SensorProps[ENUM_ZOOM].fGetSupported            = VideoProcAmpProperties[ENUM_ZOOM-NUM_VIDEOPROCAMP_ITEMS].GetSupported;
    m_SensorProps[ENUM_ZOOM].pCsPropValues            = &ZoomValues;

    m_SensorProps[ENUM_IRIS].ulCurrentValue           = IrisDefault;
    m_SensorProps[ENUM_IRIS].ulDefaultValue           = IrisDefault;
    m_SensorProps[ENUM_IRIS].pRangeNStep              = &IrisRangeAndStep[0];
    m_SensorProps[ENUM_IRIS].ulFlags                  = CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_IRIS].ulCapabilities           = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL|CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_IRIS].fSetSupported            = VideoProcAmpProperties[ENUM_IRIS-NUM_VIDEOPROCAMP_ITEMS].SetSupported;
    m_SensorProps[ENUM_IRIS].fGetSupported            = VideoProcAmpProperties[ENUM_IRIS-NUM_VIDEOPROCAMP_ITEMS].GetSupported;
    m_SensorProps[ENUM_IRIS].pCsPropValues            = &IrisValues;

    m_SensorProps[ENUM_EXPOSURE].ulCurrentValue       = ExposureDefault;
    m_SensorProps[ENUM_EXPOSURE].ulDefaultValue       = ExposureDefault;
    m_SensorProps[ENUM_EXPOSURE].pRangeNStep          = &ExposureRangeAndStep[0];
    m_SensorProps[ENUM_EXPOSURE].ulFlags              = CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_EXPOSURE].ulCapabilities       = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL|CSPROPERTY_CAMERACONTROL_FLAGS_AUTO;
    m_SensorProps[ENUM_EXPOSURE].fSetSupported        = VideoProcAmpProperties[ENUM_EXPOSURE-NUM_VIDEOPROCAMP_ITEMS].SetSupported;
    m_SensorProps[ENUM_EXPOSURE].fGetSupported        = VideoProcAmpProperties[ENUM_EXPOSURE-NUM_VIDEOPROCAMP_ITEMS].GetSupported;
    m_SensorProps[ENUM_EXPOSURE].pCsPropValues        = &ExposureValues;

    m_SensorProps[ENUM_FOCUS].ulCurrentValue          = FocusDefault;
    m_SensorProps[ENUM_FOCUS].ulDefaultValue          = FocusDefault;
    m_SensorProps[ENUM_FOCUS].pRangeNStep             = &FocusRangeAndStep[0];
    m_SensorProps[ENUM_FOCUS].ulFlags                 = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL;
    m_SensorProps[ENUM_FOCUS].ulCapabilities          = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL;
    m_SensorProps[ENUM_FOCUS].fSetSupported           = VideoProcAmpProperties[ENUM_FOCUS-NUM_VIDEOPROCAMP_ITEMS].SetSupported;
    m_SensorProps[ENUM_FOCUS].fGetSupported           = VideoProcAmpProperties[ENUM_FOCUS-NUM_VIDEOPROCAMP_ITEMS].GetSupported;
    m_SensorProps[ENUM_FOCUS].pCsPropValues           = &FocusValues;

    m_SensorProps[ENUM_FLASH].ulCurrentValue          = FlashDefault;
    m_SensorProps[ENUM_FLASH].ulDefaultValue          = FlashDefault;
    m_SensorProps[ENUM_FLASH].pRangeNStep             = &FlashRangeAndStep[0];
    m_SensorProps[ENUM_FLASH].ulFlags                 = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL;
    m_SensorProps[ENUM_FLASH].ulCapabilities          = CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL;
    m_SensorProps[ENUM_FLASH].fSetSupported           = VideoProcAmpProperties[ENUM_FLASH-NUM_VIDEOPROCAMP_ITEMS].SetSupported;
    m_SensorProps[ENUM_FLASH].fGetSupported           = VideoProcAmpProperties[ENUM_FLASH-NUM_VIDEOPROCAMP_ITEMS].GetSupported;
    m_SensorProps[ENUM_FLASH].pCsPropValues           = &FlashValues;
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    
    m_pModeVideoFormat = NULL;
    // Allocate Video Format specific array.
    m_pModeVideoFormat = new PINVIDEOFORMAT[m_ulCTypes];
    if( NULL == m_pModeVideoFormat )
    {
        DEBUGMSG( ZONE_ERROR, ( _T("PddInit:Out of Memory\r\n") ) );
        hr = ERROR_INSUFFICIENT_BUFFER;
        goto ErrorExit;
    }

    // Video Format initialization
    m_pModeVideoFormat[CAPTURE].categoryGUID         = PINNAME_VIDEO_CAPTURE;
    m_pModeVideoFormat[CAPTURE].ulAvailFormats       = 1;
    m_pModeVideoFormat[CAPTURE].pCsDataRangeVideo = new PCS_DATARANGE_VIDEO[m_pModeVideoFormat[CAPTURE].ulAvailFormats];

    if( NULL == m_pModeVideoFormat[CAPTURE].pCsDataRangeVideo )
    {
        DEBUGMSG( ZONE_ERROR, ( _T("PddInit:Out of Memory\r\n") ) );
        hr = ERROR_INSUFFICIENT_BUFFER;
        goto ErrorExit;
    }

    m_pModeVideoFormat[CAPTURE].pCsDataRangeVideo[0] = &DCAM_StreamMode_5;

    m_pModeVideoFormat[STILL].categoryGUID           = PINNAME_VIDEO_STILL;
    m_pModeVideoFormat[STILL].ulAvailFormats         = 1;
    m_pModeVideoFormat[STILL].pCsDataRangeVideo = new PCS_DATARANGE_VIDEO[m_pModeVideoFormat[STILL].ulAvailFormats];

    if( NULL == m_pModeVideoFormat[STILL].pCsDataRangeVideo )
    {
        DEBUGMSG( ZONE_ERROR, ( _T("PddInit:Out of Memory\r\n") ) );
        hr = ERROR_INSUFFICIENT_BUFFER;
        goto ErrorExit;
    }

    m_pModeVideoFormat[STILL].pCsDataRangeVideo[0]   = &DCAM_StreamMode_1;

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    m_pModeVideoCaps = NULL;
    // Allocate Video Control Caps specific array.
    m_pModeVideoCaps = new VIDCONTROLCAPS[m_ulCTypes];
    if( NULL == m_pModeVideoCaps )
    {
        DEBUGMSG( ZONE_ERROR, ( _T("PddInit:Out of Memory\r\n") ) );
        hr = ERROR_INSUFFICIENT_BUFFER;
        goto ErrorExit;
    }
    // Video Control Caps
    m_pModeVideoCaps[CAPTURE].DefaultVideoControlCaps     = DefaultVideoControlCaps[CAPTURE];
    m_pModeVideoCaps[STILL].DefaultVideoControlCaps       = DefaultVideoControlCaps[STILL];


    m_SensorModeInfo[CAPTURE].MemoryModel = CSPROPERTY_BUFFER_CLIENT_UNLIMITED;
    m_SensorModeInfo[CAPTURE].MaxNumOfBuffers = 1;
    m_SensorModeInfo[CAPTURE].PossibleCount = 1;
    m_SensorModeInfo[STILL].MemoryModel = CSPROPERTY_BUFFER_CLIENT_UNLIMITED;
    m_SensorModeInfo[STILL].MaxNumOfBuffers = 1;
    m_SensorModeInfo[STILL].PossibleCount = 1;

    m_ppModeContext = new LPVOID[m_ulCTypes];
    if ( NULL == m_ppModeContext )
    {
        DEBUGMSG( ZONE_ERROR, ( _T("PddInit:Out of Memory\r\n") ) );
        hr = ERROR_INSUFFICIENT_BUFFER;
        goto ErrorExit;
    }

ErrorExit:
    return hr;    
}



void CCameraPdd::PDD_DeInit()
{
    CameraShutdown();
    CameraDeinit();

    if( NULL != m_ppModeContext )
    {
        delete [] m_ppModeContext;
        m_ppModeContext = NULL;
    }

    if( NULL != m_pModeVideoCaps )
    {
        delete [] m_pModeVideoCaps;
        m_pModeVideoCaps = NULL;
    }

    if( NULL != m_pModeVideoFormat )
    {
        if (NULL != m_pModeVideoFormat[CAPTURE].pCsDataRangeVideo)
        {
            delete [] m_pModeVideoFormat[CAPTURE].pCsDataRangeVideo;
            m_pModeVideoFormat[CAPTURE].pCsDataRangeVideo = NULL;
        }

        if (NULL != m_pModeVideoFormat[STILL].pCsDataRangeVideo)
        {
            delete [] m_pModeVideoFormat[STILL].pCsDataRangeVideo;
            m_pModeVideoFormat[STILL].pCsDataRangeVideo = NULL;
        }

        delete [] m_pModeVideoFormat;
        m_pModeVideoFormat = NULL;
    }
}



DWORD CCameraPdd::GetAdapterInfo( PADAPTERINFO pAdapterInfo )
{
    pAdapterInfo->ulCTypes = m_ulCTypes;
    pAdapterInfo->PowerCaps = PowerCaps;

    //Camera MDD and DShow support DRIVER_VERSION and DRIVER_VERSION_2. Defined in camera.h
    pAdapterInfo->ulVersionID = DRIVER_VERSION_2; 
    memcpy( &pAdapterInfo->SensorProps, &m_SensorProps, sizeof(m_SensorProps));

    return ERROR_SUCCESS;

}

DWORD CCameraPdd::HandleVidProcAmpChanges( DWORD dwPropId, LONG lFlags, LONG lValue )
{
    PSENSOR_PROPERTY pDevProp = NULL;

    pDevProp = m_SensorProps + dwPropId;
    
    if( CSPROPERTY_VIDEOPROCAMP_FLAGS_MANUAL == lFlags )
    {
        pDevProp->ulCurrentValue = lValue;
    }

    pDevProp->ulFlags = lFlags;
    return ERROR_SUCCESS;
}

DWORD CCameraPdd::HandleCamControlChanges( DWORD dwPropId, LONG lFlags, LONG lValue )
{
    PSENSOR_PROPERTY pDevProp = NULL;
    
    pDevProp = m_SensorProps + dwPropId;
    
    if( CSPROPERTY_CAMERACONTROL_FLAGS_MANUAL == lFlags )
    {
        pDevProp->ulCurrentValue = lValue;
    }

    pDevProp->ulFlags = lFlags;
    return ERROR_SUCCESS;
}

DWORD CCameraPdd::HandleVideoControlCapsChanges( LONG lModeType ,ULONG ulCaps )
{
    m_pModeVideoCaps[lModeType].CurrentVideoControlCaps = ulCaps;
    return ERROR_SUCCESS;
}

DWORD CCameraPdd :: SetPowerState( CEDEVICE_POWER_STATE PowerState )
{
    DWORD dwErr = ERROR_SUCCESS;

    switch (PowerState)
    {
    case D0:
        RETAILMSG(ZONE_WARN, (TEXT("CAM: D0\r\n")));

        if (m_PowerState != D0 && m_PowerState != D2) 
            ResumeCamera();

        m_PowerState = D0;

        dwErr = ERROR_SUCCESS;
        break;

    case D2:
        RETAILMSG(ZONE_WARN, (TEXT("CAM: D2\r\n")));

        if (m_PowerState != D0 && m_PowerState != D2) 
            ResumeCamera();

        m_PowerState = D2;

        dwErr = ERROR_SUCCESS;
        break;

    case D3:
        RETAILMSG(ZONE_WARN, (TEXT("CAM: D3\r\n")));
        m_PowerState = D3;

        SuspendCamera();
        dwErr = ERROR_SUCCESS;
        break;
        
    case D4:
        RETAILMSG(ZONE_WARN, (TEXT("CAM: D4\r\n")));
        m_PowerState = D4;

        SuspendCamera();
        dwErr = ERROR_SUCCESS;
        break;

    default:
        break;
    }

    return dwErr;
}

DWORD CCameraPdd::HandleAdapterCustomProperties( PUCHAR pInBuf, DWORD  InBufLen, PUCHAR pOutBuf, DWORD  OutBufLen, PDWORD pdwBytesTransferred )
{
     DEBUGMSG( ZONE_IOCTL, ( _T("IOControl Adapter PDD: Unsupported PropertySet Request\r\n")) );
    return ERROR_NOT_SUPPORTED;
}

DWORD CCameraPdd::InitSensorMode( ULONG ulModeType, LPVOID ModeContext )
{
    DWORD hr = ERROR_SUCCESS;
    
    ASSERT( ModeContext );

    // Prepare buffers here for Preview and Still mode.
    // Create DMA descriptors and chain them.

    if (ulModeType == CAPTURE)
    {
        if (!AllocateHWBuffers(VIDEO_CAPTURE_BUFFER) )
        {
            RETAILMSG(ZONE_ERROR, ( _T("InitSensorMode: AllocateHWBuffers() failed\r\n") ) );
            hr = ERROR_INSUFFICIENT_BUFFER;
            goto ErrorExit;
        }
    }
    else if (ulModeType == STILL)
    {
        if (!AllocateHWBuffers(STILL_CAPTURE_BUFFER))
        {
            RETAILMSG(ZONE_ERROR, ( _T("InitSensorMode: AllocateHWBuffers() failed\r\n") ) );
            hr = ERROR_INSUFFICIENT_BUFFER;
            goto ErrorExit;
        }
    }
    else
    {
        hr = ERROR_INVALID_PARAMETER;
        goto ErrorExit;

    }

    m_ppModeContext[ulModeType] = ModeContext;

ErrorExit:
    return hr;
}

DWORD CCameraPdd::DeInitSensorMode( ULONG ulModeType )
{
    DWORD hr = ERROR_SUCCESS;
    
    if (ulModeType == CAPTURE)
    {
        if (!DeAllocateHWBuffers(VIDEO_CAPTURE_BUFFER) )
        {
            RETAILMSG(ZONE_ERROR, ( _T("DeInitSensorMode: DeAllocateHWBuffers() failed\r\n") ) );
            goto ErrorExit;
        }
    }
    else if (ulModeType == STILL)
    {
        if (!DeAllocateHWBuffers(STILL_CAPTURE_BUFFER))
        {
            RETAILMSG(ZONE_ERROR, ( _T("DeInitSensorMode: DeAllocateHWBuffers() failed\r\n") ) );
            goto ErrorExit;
        }
    }
    else
    {
        hr = ERROR_INVALID_PARAMETER;
        goto ErrorExit;
    }

ErrorExit:
    return hr;
}

DWORD CCameraPdd::SetSensorState( ULONG lModeType, CSSTATE csState )
{
    DWORD dwError = ERROR_SUCCESS;

    switch ( csState )
    {
        case CSSTATE_STOP:
            m_CsState[lModeType] = CSSTATE_STOP;
            if( (m_CsPrevState[lModeType] == CSSTATE_RUN) &&
                (lModeType == CAPTURE))
            {
                StopHWCapture( );
            }
            m_CsPrevState[lModeType] = CSSTATE_STOP;


            break;

        case CSSTATE_PAUSE:
            m_CsState[lModeType] = CSSTATE_PAUSE;
            if( (m_CsPrevState[lModeType] == CSSTATE_RUN) &&
                (lModeType == CAPTURE))
            {
                StopHWCapture( );
            }
            m_CsPrevState[lModeType] = CSSTATE_PAUSE;
            break;

        case CSSTATE_RUN:
            ASSERT(lModeType == CAPTURE);
            if (!m_bStillInProgress)
            {

                m_CsState[lModeType] = CSSTATE_RUN;
                m_CsPrevState[lModeType] = CSSTATE_RUN;
                
                if (!StartHWCapture())
                {
                    RETAILMSG(ZONE_ERROR, ( _T("SetSensorState: StartHWCapture() failed\r\n") ) );
                }
            }
            break;

        default:
            DEBUGMSG( ZONE_IOCTL|ZONE_ERROR, ( _T("IOControl(%08x): Incorrect State\r\n"), this ) );
            dwError = ERROR_INVALID_PARAMETER;
    }

    return dwError;
}

DWORD CCameraPdd::TakeStillPicture( LPVOID pBurstModeInfo )
{
    DWORD dwError = ERROR_SUCCESS;
    //Ignore pBurstModeInfo
    m_CsState[STILL] = CSSTATE_RUN;
    m_bStillInProgress = TRUE;
    PauseCapture();
    if (!CaptureHWStillImage())
    {
        RETAILMSG(ZONE_ERROR, ( _T("TakeStillPicture: CaptureHWStillImage() failed\r\n") ) );
    }
    return dwError;
}

bool
CCameraPdd::PauseCapture( void )
{
    if (m_CsState[CAPTURE]== CSSTATE_RUN)
    {
        SetSensorState(CAPTURE, CSSTATE_PAUSE);
    }
    return true;
}


DWORD CCameraPdd::GetSensorModeInfo( ULONG ulModeType, PSENSORMODEINFO pSensorModeInfo )
{
    pSensorModeInfo->MemoryModel = m_SensorModeInfo[ulModeType].MemoryModel;
    pSensorModeInfo->MaxNumOfBuffers = m_SensorModeInfo[ulModeType].MaxNumOfBuffers;
    pSensorModeInfo->PossibleCount = m_SensorModeInfo[ulModeType].PossibleCount;
    pSensorModeInfo->VideoCaps.DefaultVideoControlCaps = DefaultVideoControlCaps[ulModeType];
    pSensorModeInfo->VideoCaps.CurrentVideoControlCaps = m_pModeVideoCaps[ulModeType].CurrentVideoControlCaps;
    pSensorModeInfo->pVideoFormat = &m_pModeVideoFormat[ulModeType];
    
    return ERROR_SUCCESS;
}

DWORD CCameraPdd::SetSensorModeFormat( ULONG ulModeType, PCS_DATARANGE_VIDEO pCsDataRangeVideo )
{
    // MDD will take care of ulModeType. It will never be out of range. MDD will also ask for
    // the format we support. We need not to chech it here. 
    // As we support only one mode for each type(Still and Preview), we need not to change 
    // the format. These formats are already set.

    memcpy( &m_CurrentFormat[ulModeType], pCsDataRangeVideo, sizeof ( CS_DATARANGE_VIDEO ) );

    return ERROR_SUCCESS;
}

PVOID CCameraPdd::AllocateBuffer( ULONG ulModeType )
{  
    // In this PDD, we dont need it
    return NULL;
}

DWORD CCameraPdd::DeAllocateBuffer( ULONG ulModeType, PVOID pBuffer )
{
    return ERROR_SUCCESS;
}

DWORD CCameraPdd::RegisterClientBuffer( ULONG ulModeType, PVOID pBuffer )
{
    // In this PDD, we dont need it.
    return ERROR_SUCCESS;
}

DWORD CCameraPdd::UnRegisterClientBuffer( ULONG ulModeType, PVOID pBuffer )
{
    // DShow is not going to use pBuffer (which was originally allocated by DShow) anymore. If the PDD
    // is keeping a cached pBuffer pointer (in RegisterClientBuffer()) then this is the right place to
    // stop using it and maybe set the cached pointer to NULL. 
    // Note: PDD must not delete this pointer as it will be deleted by DShow itself
    // This PDD does not use any cached pointers.
    return ERROR_SUCCESS;
}

DWORD CCameraPdd::HandleSensorModeCustomProperties( ULONG ulModeType, PUCHAR pInBuf, DWORD  InBufLen, PUCHAR pOutBuf, DWORD  OutBufLen, PDWORD pdwBytesTransferred )
{
    DEBUGMSG( ZONE_IOCTL, ( _T("IOControl: Unsupported PropertySet Request\r\n")) );
    return ERROR_NOT_SUPPORTED;
}

bool CCameraPdd::AllocateHWBuffers( ULONG ucBufferType )
{
    P_CAMERA_DMA_BUFFER_INFO pCamDmaBuf;

    // Allocate buffers for motion video and still image
    for (int i = 0; i < (VIDEO_CAPTURE_BUFFER == ucBufferType ? MAX_HW_FRAMES : 1); i++)
    {
        pCamDmaBuf = VIDEO_CAPTURE_BUFFER == ucBufferType ? &(m_CameraHWVideoBuffers[i]) : &m_CameraHWStillBuffer;

        // Set the size field to 0x0 so we can query to find what size we need
        // The driver will fill in the size field for us.
        // Then add 4K to that so that we have room to 4K align the buffer.
        pCamDmaBuf->size = 0;
        CameraPrepareBuffer(pCamDmaBuf, ucBufferType);

        // Must 4K align the memory
        pCamDmaBuf->pBuf = reinterpret_cast<DWORD *>(malloc(pCamDmaBuf->size + 4096));

        if (pCamDmaBuf->pBuf == 0x0)
        {
            DEBUGMSG(ZONE_FUNCTION|ZONE_ERROR, (_T("AllocateHWBuffers: Not enough memory to allocate %d bytes for Camera Buffer.\r\n"), pCamDmaBuf->size)) ;
            return false;
        }

        pCamDmaBuf->VirtAddr = reinterpret_cast<DWORD>(pCamDmaBuf->pBuf);

        // Align the buffer to 4K boundary
        if (pCamDmaBuf->VirtAddr & 0xFFF)
        {
            pCamDmaBuf->VirtAddr += 0x1000 - (pCamDmaBuf->VirtAddr & 0xFFF);
        }

        // Now that we have allocated enough memory, go ahead and prepare the buffer
        if (!CameraPrepareBuffer(pCamDmaBuf, ucBufferType))
        {
            DEBUGMSG(ZONE_FUNCTION|ZONE_ERROR, (_T("AllocateHWBuffers: Unable to prepare DMA buffer.\r\n"), pCamDmaBuf->size)) ;
            return false;
        }

        // Set the DMA buffer memory to uncacheable, unbufferable
        VirtualSetAttributes(reinterpret_cast<void *>(pCamDmaBuf->VirtAddr), pCamDmaBuf->size, 0, 0xC, NULL);

        DEBUGMSG(ZONE_INIT, (_T("AllocateHWBuffers: Buffer allocated for %s pin [0x%08x - %d bytes].\r\n"),\
                 STILL_CAPTURE_BUFFER == ucBufferType ? L"still" : L"capture/preview", pCamDmaBuf->VirtAddr, pCamDmaBuf->size));
    }

    return true;
}

bool CCameraPdd::DeAllocateHWBuffers( ULONG ucBufferType )
{
    P_CAMERA_DMA_BUFFER_INFO pCamDmaBuf;

    // DeAllocate buffers for motion video and still image
    for (int i = 0; i < (VIDEO_CAPTURE_BUFFER == ucBufferType ? MAX_HW_FRAMES : 1); i++)
    {
        pCamDmaBuf = VIDEO_CAPTURE_BUFFER == ucBufferType ? &(m_CameraHWVideoBuffers[i]) : &m_CameraHWStillBuffer;

        if (pCamDmaBuf->pBuf)
        {
            CameraUnprepareBuffer(pCamDmaBuf, ucBufferType);
            free(pCamDmaBuf->pBuf);
            pCamDmaBuf->pBuf = NULL;

            DEBUGMSG(ZONE_INIT, (_T("DeAllocateHWBuffers: Buffer deallocated for %s pin [0x%08x - %d bytes].\r\n"),\
                     STILL_CAPTURE_BUFFER == ucBufferType ? L"still" : L"capture/preview", pCamDmaBuf->VirtAddr, pCamDmaBuf->size));
        }
    }
    
    return true;
}

bool CCameraPdd::GetCurrentHWBuffer(
    ULONG ulPinId,
    P_CAMERA_DMA_BUFFER_INFO pCamDmaBuffer
    )
{
    bool fRet = false;
    if( NULL != pCamDmaBuffer)
    {
        if (CeSafeCopyMemory( pCamDmaBuffer, (STILL == ulPinId) ? &m_CameraHWStillBuffer : &m_CameraHWVideoBuffers[m_ulCurrentFrame],
                              sizeof(CAMERA_DMA_BUFFER_INFO)))
            fRet = true;
    }
    return fRet;
}

bool CCameraPdd::StartHWCapture( )
{
    if (!m_bCameraHWRunning)
    {
        // Mainstone II: Start streaming video out of sensor
        DEBUGMSG( ZONE_WARN, ( _T("StartHWCapture(%08x): run camera sensor\r\n"), this ) );

        // Reset current frame counter
        m_ulCurrentFrame = 0;

        // Submit the h/w dma buffers to the camera driver.
        for (int i = 0; i < MAX_HW_FRAMES; i++)
        {           
            CameraSubmitBuffer(&m_CameraHWVideoBuffers[i], VIDEO_CAPTURE_BUFFER);
        }

        CameraStartVideoCapture();

        m_bCameraHWRunning = true;
    }

    return true;
}


bool CCameraPdd::StopHWCapture( )
{
    if (m_bCameraHWRunning)
    {
        // Mainstone II: Stop streaming video out of sensor
        DEBUGMSG( ZONE_WARN, ( _T("StopHWCapture(%08x): stop camera sensor\r\n"), this ) );

        CameraStopVideoCapture();

        m_bCameraHWRunning = false;
    }

    return true;
}

        
bool CCameraPdd::CaptureHWStillImage( )
{
    // Mainstone II: Grab still image out of sensor
    DEBUGMSG( ZONE_WARN, ( _T("CaptureHWStillImage(%08x): grab still image from sensor\r\n"), this ) );

    // Submit the h/w dma buffers to the camera driver.
    CameraSubmitBuffer(&m_CameraHWStillBuffer, STILL_CAPTURE_BUFFER);

//    m_CsState[CAPTURE] = CSSTATE_PAUSE;

    CameraCaptureStillImage();

    return true;
}

bool CCameraPdd::RevertCaptureState( void )
{
    SetSensorState(CAPTURE, m_CsPrevState[CAPTURE]);
    return true;
}

void CCameraPdd::CameraHandleVideoFrame ( )
{
    // Here Call MDD_HandleIO which will inturn call
    // FillBuffer.
    MDD_HandleIO( m_ppModeContext[CAPTURE], CAPTURE );
    
    // After that Resubmit buffer back
    CameraSubmitBuffer(&m_CameraHWVideoBuffers[m_ulCurrentFrame], VIDEO_CAPTURE_BUFFER);

    // Maintain the frame counter
    m_ulCurrentFrame = (m_ulCurrentFrame + 1) % MAX_HW_FRAMES;
}

void CCameraPdd::CameraHandleStillFrame ( )
{
    // Here Call MDD_HandleIO which will inturn call
    // FillBuffer.
    MDD_HandleIO( m_ppModeContext[STILL], STILL );
    m_bStillInProgress = FALSE;

    // As this function is called in response to TakeStillPicture,
    // it should restore the capture state;

     RevertCaptureState();
}


DWORD CCameraPdd::FillBuffer( ULONG ulModeType, PUCHAR pImage )
{
    DWORD dwRet = 0;
    // This function will be called in response to MDD_HandleIO 

    // If any processing is required (e.g. ColorConversion), it
    // should be done here.
    PCS_VIDEOINFOHEADER pCsVideoInfoHdr = &m_CurrentFormat[ulModeType].VideoInfoHeader;
    
    
    ASSERT(pCsVideoInfoHdr->bmiHeader.biSizeImage != 0);

    // MDD will make sure that the buffer is sufficient for the image.   

    if (ulModeType == CAPTURE)
    {
        dwRet = pCsVideoInfoHdr->bmiHeader.biSizeImage; 
        CUYVYToYV12Convertor((BYTE *)m_CameraHWVideoBuffers[m_ulCurrentFrame].VirtAddr, (BYTE *)pImage);
    }
    else if (ulModeType == STILL)
    {
        dwRet = pCsVideoInfoHdr->bmiHeader.biSizeImage; 
        memcpy(pImage, (void *)m_CameraHWStillBuffer.VirtAddr, dwRet);
    }

    // return the size of the image filled
    return dwRet;
}


void CCameraPdd::SuspendCamera( )
{
    if(m_bCameraHWRunning)
    {
        m_bCameraWasRunning = TRUE;
    }

    PauseCapture();
    StopHWCapture();
    CameraSleep();
}

void CCameraPdd::ResumeCamera( )
{
    // Restart camera sensor if it was running before
    if ( m_bCameraWasRunning )
    {
        CameraResume();
        RevertCaptureState();
        m_bCameraWasRunning = FALSE;
        StartHWCapture();
    }
}

void CameraVideoFrameCallback( DWORD dwContext )
{
    CCameraPdd * pCamDevice = reinterpret_cast<CCameraPdd *>( dwContext );
    
    // Video frame is ready - put it into stream
    
    if (NULL != pCamDevice)
    {
        pCamDevice->CameraHandleVideoFrame();
    }
}

void CameraStillFrameCallback( DWORD dwContext )
{
    CCameraPdd * pCamDevice = reinterpret_cast<CCameraPdd *>( dwContext );
    
    // Still image frame is ready - put it into stream

    if (NULL != pCamDevice)
    {
        pCamDevice->CameraHandleStillFrame();
    }
}

extern "C"
int SensorInitPlatform(
    P_XLLP_Camera_Context_T pCameraContext,
    P_XLLP_Camera_DMA_Context_T pDmaContext
    )
{
//    PHYSICAL_ADDRESS PA;
//    MAINSTONEII_BLR_REGS        *pBLR = NULL;

    RETAILMSG(1, (TEXT("CameraInitPlatform\r\n")));

    // Mainstone II specific memory allocation
//    PA.QuadPart  = MAINSTONEII_BASE_REG_PA_FPGA;
//    pBLR = (MAINSTONEII_BLR_REGS *) MmMapIoSpace(PA, sizeof(MAINSTONEII_BLR_REGS), FALSE);

/*    if(pBLR)
    {
        // os mapped register address   
        pCameraContext->board_reg_base = (unsigned int)pBLR;
    } else*/ {
        return FALSE;
    }

    // Camera sensor specific information (Agilent ADCM2650)
    pCameraContext->sensor_type = XLLP_CAMERA_TYPE_ADCM_2650; 

    pCameraContext->camera_functions->init = CameraFuncADCM2650Init;
    pCameraContext->camera_functions->deinit = CameraFuncADCM2650DeInit;
    pCameraContext->camera_functions->set_capture_format = CameraFuncADCM2650SetCaptureFormat;
    pCameraContext->camera_functions->start_capture = CameraFuncADCM2650StartCapture;
    pCameraContext->camera_functions->stop_capture = CameraFuncADCM2650StopCapture;

    // capture image info
    pCameraContext->capture_width = DEFAULT_VIDEO_WIDTH;
    pCameraContext->capture_height = DEFAULT_VIDEO_HEIGHT;
    pCameraContext->Video_capture_width = DEFAULT_VIDEO_WIDTH;
    pCameraContext->Video_capture_height = DEFAULT_VIDEO_HEIGHT;
    pCameraContext->Still_capture_width = DEFAULT_STILL_WIDTH;
    pCameraContext->Still_capture_height = DEFAULT_STILL_HEIGHT;
    pCameraContext->sensor_flip_mode = XLLP_CAMERA_VIDEO_FLIP_VERTICAL |
                                       XLLP_CAMERA_STILL_FLIP_HORIZONTAL;
    pCameraContext->Video_capture_input_format = DEFAULT_CAMERA_VIDEO_INPUT_FORMAT;
    pCameraContext->Video_capture_output_format = DEFAULT_CAMERA_VIDEO_OUTPUT_FORMAT;

    pCameraContext->Still_capture_input_format = DEFAULT_CAMERA_STILL_INPUT_FORMAT;
    pCameraContext->Still_capture_output_format = DEFAULT_CAMERA_STILL_OUTPUT_FORMAT;

    // Assign the DMA channels to camera sensor
    pDmaContext->dma_channels[0] = XLLP_DMAC_CHANNEL_16;
    pDmaContext->dma_channels[1] = XLLP_DMAC_CHANNEL_17;
    pDmaContext->dma_channels[2] = XLLP_DMAC_CHANNEL_18;

    return TRUE;
}

extern "C"
int SensorDeinitPlatform(
    P_XLLP_Camera_Context_T pCameraContext,
    P_XLLP_Camera_DMA_Context_T pDmaContext
    )
{
    pDmaContext->dma_channels[0] = (XLLP_DMAC_CHANNEL_T)0xFF;
    pDmaContext->dma_channels[1] = (XLLP_DMAC_CHANNEL_T)0xFF;
    pDmaContext->dma_channels[2] = (XLLP_DMAC_CHANNEL_T)0xFF;

    if(pCameraContext->board_reg_base)
    {
//        MmUnmapIoSpace((void *)pCameraContext->board_reg_base, sizeof(MAINSTONEII_BLR_REGS));
        pCameraContext->board_reg_base = 0;
    }
    return TRUE;
}
// This routine converts UYVY into YV12. It is used only for Capture pin.


struct UYVYMacroPixel
{
    BYTE U0;
    BYTE Y0;
    BYTE V0;
    BYTE Y1;
};


DWORD CCameraPdd:: CUYVYToYV12Convertor(BYTE *pInput, BYTE *pOutput)
{ 

    PCS_VIDEOINFOHEADER pCsVideoInfoHdr = &m_CurrentFormat[CAPTURE].VideoInfoHeader;

    ASSERT(pCsVideoInfoHdr != NULL);

    LONG lHeight = abs(pCsVideoInfoHdr->bmiHeader.biHeight);
    LONG lWidth = pCsVideoInfoHdr->bmiHeader.biWidth;


    LONG SrcStride = WIDTHBYTES(lWidth *16); // Input is 16 bit YUV packed (UYVY)
    LONG DstStride = lWidth; // As output is planar 
    
    LONG lSrcDelta = SrcStride - (lWidth * 2);
    LONG lDstDelta = 0; 
    

    if (0 == lHeight)
        return ERROR_SUCCESS;

    BYTE *pYUY2Line1 = pInput;
    BYTE *pYUY2Line2;

    BYTE *pDstYLine1 = pOutput;
    BYTE *pDstYLine2 = pOutput + DstStride;
    BYTE *pDstV = pOutput + (DstStride * lHeight);
    BYTE *pDstU = pDstV + ((DstStride * lHeight) >> 2);

    do {
        // YUY2 images can have odd height
        pYUY2Line2 = (1 == lHeight) ? pYUY2Line1 
                                    : (pYUY2Line1 + SrcStride);
        pDstYLine2 = pDstYLine1 + DstStride;

        // add two because we'll pre-decrement in the do-loop below
        LONG lDecWidth = lWidth + 2;

        // read/process one macropixel (2 pixels) at a time
        while (lDecWidth -= 2)
        {
            UYVYMacroPixel mpSrc1, mpSrc2;
            mpSrc1.U0 = *pYUY2Line1++;
            mpSrc1.Y0 = *pYUY2Line1++;
            mpSrc1.V0 = *pYUY2Line1++;
            mpSrc1.Y1 = *pYUY2Line1++;
            mpSrc2.U0 = *pYUY2Line2++;
            mpSrc2.Y0 = *pYUY2Line2++;
            mpSrc2.V0 = *pYUY2Line2++;
            mpSrc2.Y1 = *pYUY2Line2++;

            // Y plane
            *pDstYLine1++ = mpSrc1.Y0;
            *pDstYLine1++ = mpSrc1.Y1;
            *pDstYLine2++ = mpSrc2.Y0;
            *pDstYLine2++ = mpSrc2.Y1;

            // V plane
            *pDstV++ = (mpSrc1.V0 + mpSrc2.V0) >> 1;

            // U plane
            *pDstU++ = (mpSrc1.U0 + mpSrc2.U0) >> 1;
        }

        pYUY2Line1 = pYUY2Line2 + lSrcDelta;
        // pYUY2Line2 gets set at the top of the loop, based on pYUY2Line1
        pDstYLine1 = pDstYLine2 + lDstDelta;
        // pDstYLine2 gets set at the top of the loop, based on pDstYLine1
        pDstV      += lDstDelta >> 1;
        pDstU      += lDstDelta >> 1;
    } while (lHeight -= 2);

    return ERROR_SUCCESS;
}


