//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
//
/******************************************************************************
**
** Copyright 2001, 2002 Intel Corporation. All Rights Reserved.
**
** Portions of the source code contained or described herein and all documents
** related to such source code (Material) are owned by Intel Corporation
** or its suppliers or licensors and is licensed by Microsoft Corporation for distribution.  
** Title to the Material remains with Intel Corporation or its suppliers and licensors. 
** Use of the Materials is subject to the terms of the Microsoft license agreement which accompanied the Materials.  
** No other license under any patent, copyright, trade secret or other intellectual
** property right is granted to or conferred upon you by disclosure or
** delivery of the Materials, either expressly, by implication, inducement,
** estoppel or otherwise 
** Some portion of the Materials may be copyrighted by Microsoft Corporation.
**
**  FILENAME:       xllp_dmac.c
**
**  PURPOSE: contains all primitive functions for Bulverde DMA Controller register 
**           access and control
**                  
******************************************************************************/
#include <windows.h>
#include <bulverde_intr.h>
#include <bulverde_dma.h>
#include <xllp_dmac.h>

#define XLLP_DMAC_DDADR_RESERVED_MASK	 0xFFFFFFF0
#define XLLP_DMAC_DRCMR_ENABLE     	 0x80
#define XLLP_DMAC_DRCMR_DISABLE    	 0x00

volatile P_XLLP_DMAC_T	pDmacHandle;

/**
 * Channel priority sets
 **/
const XLLP_DMAC_CHANNEL_T aPrioritySetCount = 8;
const XLLP_DMAC_CHANNEL_T ChannelPriorityHigh[]=
{
    XLLP_DMAC_CHANNEL_0,
    XLLP_DMAC_CHANNEL_1,
    XLLP_DMAC_CHANNEL_2,
    XLLP_DMAC_CHANNEL_3,
    XLLP_DMAC_CHANNEL_16,
    XLLP_DMAC_CHANNEL_17,
    XLLP_DMAC_CHANNEL_18,
    XLLP_DMAC_CHANNEL_19
};    
const XLLP_DMAC_CHANNEL_T ChannelPriorityMedium[]=
{
    XLLP_DMAC_CHANNEL_4,
    XLLP_DMAC_CHANNEL_5,
    XLLP_DMAC_CHANNEL_6,
    XLLP_DMAC_CHANNEL_7,
    XLLP_DMAC_CHANNEL_20,
    XLLP_DMAC_CHANNEL_21,
    XLLP_DMAC_CHANNEL_22,
    XLLP_DMAC_CHANNEL_23
};    
const XLLP_DMAC_CHANNEL_T ChannelPriorityLow[]=
{
    XLLP_DMAC_CHANNEL_8,
    XLLP_DMAC_CHANNEL_9,
    XLLP_DMAC_CHANNEL_10,
    XLLP_DMAC_CHANNEL_11,
    XLLP_DMAC_CHANNEL_24,
    XLLP_DMAC_CHANNEL_25,
    XLLP_DMAC_CHANNEL_26,
    XLLP_DMAC_CHANNEL_27
};    
const XLLP_DMAC_CHANNEL_T ChannelPriorityLowest[]=
{
    XLLP_DMAC_CHANNEL_12,
    XLLP_DMAC_CHANNEL_13,
    XLLP_DMAC_CHANNEL_14,
    XLLP_DMAC_CHANNEL_15,
    XLLP_DMAC_CHANNEL_28,
    XLLP_DMAC_CHANNEL_29,
    XLLP_DMAC_CHANNEL_30,
    XLLP_DMAC_CHANNEL_31
};    

// This is only needed when running without the XLLP DMA Engine.
XLLP_STATUS_T XllpDmacInitHandle(P_XLLP_DMAC_T address)
{
	pDmacHandle = address;
	return 0;
}


/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacFillLinkedDesc

  Description: XllpDmacFillLinkedDesc is used to setup a descriptor or chained 
  			   descriptors for DMA transfer operations. To setup a descriptor 
  			   (ie. no chaining), the pointer to the next descriptor pNextDesc 
  			   is expected to be zero, otherwise pNextDesc expected to be a 
  			   valid descriptor pointer.  Users wanting to chain multiple 
  			   descriptors will call this function to accomplish that task.
  			   
  Global Registers Modified: 
	None
		
  Input Arguments:
	pDesc: Hold the pointer to the descriptor to be configured, cannot be a 
		   null pointer.
	pNextDesc: Holds the pointer to the next descriptor used in the DMA transfer
			   process after completely transferring data in the current descriptor
			   pDesc if the channel is not programmed to be stop.  pNextDesc can 
			   be a null pointer only if this descriptor is not meant to be chained.
	aStopContinue:  Specifies whether or not to configure this descriptor to stop
				    the channel after completely transferring data in this descriptor.
	aBranch: Specifies whether to configure this descriptor to enable or disable 
			 the descriptor branching mechanism.  Valid values are element of type
			 XLLP_DMAC_DESC_BRANCH_T (1 or 0).
	aSrcAddr: Holds the source address for this descriptor, cannot be null.
	aTargetAddr: Holds the target address for this descriptor, cannot be null.
	pCmd: Holds the commands for this descriptor.  Users are required to initialize 
		  the various fields of pCmd data structure before making this call.
	                 
  Output Arguments:
  	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacFillLinkedDesc(
						   P_XLLP_DMAC_DESCRIPTOR_T pDesc,
						   P_XLLP_DMAC_DESCRIPTOR_T pNextDescPhyAddr,
						   XLLP_DMAC_DESC_ENABLE_T  aStopContinue,
						   XLLP_DMAC_DESC_BRANCH_T  aBranch,
						   XLLP_UINT32_T            aSrcAddr,
						   XLLP_UINT32_T            aTargetAddr,
						   XLLP_DMAC_COMMAND_T*     pCmd
						  )
{
	XLLP_VUINT32_T aTargetValue;

	
	/* Construct command value from pCmd  */
	XLLP_UINT32_T aCommand =  (XLLP_UINT32_T)(
								( pCmd->aLen) | ((pCmd->aWidth) << XLLP_BIT_FIELD_14)|
								((pCmd->aSize) << XLLP_BIT_FIELD_16)      |
								((pCmd->aEndian) << XLLP_BIT_FIELD_18)    | 
								((pCmd->aFlyByT) << XLLP_BIT_FIELD_19)    |
								((pCmd->aFlyByS) << XLLP_BIT_FIELD_20)    |
								((pCmd->aEndIrqEn)   << XLLP_BIT_FIELD_21)|
								((pCmd->aStartIrqEn) << XLLP_BIT_FIELD_22)|
								((pCmd->aAddrMode)   << XLLP_BIT_FIELD_23)|
								((pCmd->aCmpEn)   << XLLP_BIT_FIELD_25)   |
								((pCmd->aFlowTrg) << XLLP_BIT_FIELD_28)   |
								((pCmd->aFlowSrc) << XLLP_BIT_FIELD_29)   |
								((pCmd->aIncTrgAddr) << XLLP_BIT_FIELD_30)|
								((pCmd->aIncSrcAddr) << XLLP_BIT_FIELD_31)
							   );	   	

	/* Clear reserved bit fields of DDADR */
	XLLP_VUINT32_T aNextDesc = (XLLP_UINT32_T)pNextDescPhyAddr;  
	aNextDesc = (XLLP_UINT32_T)(aNextDesc & XLLP_DMAC_DDADR_RESERVED_MASK);
	aTargetValue = (XLLP_UINT32_T)(aNextDesc + (aBranch<<XLLP_BIT_FIELD_1) + aStopContinue);

	/* Fill descriptor entries            */
	pDesc->DDADR = (XLLP_UINT32_T)(aNextDesc) + (aBranch<<XLLP_BIT_FIELD_1) + aStopContinue;
	pDesc->DSADR = (XLLP_UINT32_T)aSrcAddr;
	pDesc->DTADR = (XLLP_UINT32_T)aTargetAddr;
	pDesc->DCMD  = (XLLP_UINT32_T)aCommand;

}


/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacCfgChannelDescTransfer

  Description: XllpDmacCfgChannelDescTransfer configures the specified DMA 
  			   channel for a Descriptor Mode transfer operation.  To configure 
  			   a channel for a no descriptor mode transfer, use the 
  			   XllpDmacCfgChannelNoDescTransfer primitive.  It is expected that 
  			   the developer has already made calls to XllpDmacAllocChannel for 
  			   an available channel and bound it to a service routine for interrupt
  			   that may occur on the channel, and XllpDmacFillLinkedDesc to 
  			   configure the descriptor before calling this primitives.
  			   
  Global Registers Modified: 
	DDADRx, DRCMRx registers.  DSADRx, DTADRx, and DCMDx register will be modified
	when the descriptor is loaded.  x indicates the channel number		
  
  Input Arguments:
	pDesc:  Hold the pointer to the descriptor to be used in configuring the 
			specified DMA channel.  The pointer cannot be null.
	aChannel:  Specifies the channel to be configured for the DMA transfer.
	aDeviceDrcmr:  Specifies the device to be mapped to the DMA channel for the transfer
			  	   operation.
	aLignment: Indicated whether to enable byte alignment for memory accesses			  
	                 
  Output Arguments:
  	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacCfgChannelDescTransfer(
					    		   P_XLLP_DMAC_DESCRIPTOR_T  pDesc,
								   XLLP_DMAC_CHANNEL_T       aChannel,
								   //XLLP_DMAC_DRCMR_T         aDeviceDrcmr,
								   XLLP_DMAC_DEVICE_T        aDeviceDrcmr,
								   XLLP_DMAC_ALIGNMENT_T     aLignment
				     	  		   )
{
	XLLP_VUINT32_T aTargetData; 

	if (aLignment == XLLP_DMAC_ALIGNMENT_ON)
	{
		aTargetData = pDmacHandle->DALGN;
		aTargetData |= (1 << aChannel);
		pDmacHandle->DALGN = aTargetData;
	}

	/* Map device to channel                 */
	/* No mapping required for mem2mem moves */
	if (aDeviceDrcmr != XLLP_DMAC_MEM2MEM_MOVE)
	{
		XllpDmacMapDeviceToChannel(aDeviceDrcmr, aChannel);
	}

	pDmacHandle->DDG[aChannel].DDADR = (XLLP_UINT32_T)pDesc;
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacCfgChannelNoDescTransfer

  Description: XllpDmacCfgChannelNoDescTransfer configures the specified DMA 
  			   channel for a No Descriptor Mode transfer operation.  To configure
  			   a channel for a descriptor mode transfer, use the 
  			   XllpDmacCfgChannelDescTransfer primitive.  It is expected that the
  			   developer has already obtained an available channel using the
  			   XllpDmacAllocChannel primitive and bound it to a service routine 
  			   for interrupt that may occur on the channel.
  			   
  Global Registers Modified: 
	DRCMRx, DSADRx, DTADRx, DCSRx, and DCMDx registers, where x indicate the 
	channel number  
  
  Input Arguments:
	aSourceAddr:  Source address of  the DMA transfer to be configured 
	aTargetAddr:  Target address of the DMA transfer to be configured
	pCmd:  Commands for the channel to be configured.  Users are required to 
		   initialize the various fields of pCmd data structure before making this
		   call.
	aChannel: Specifies the channel to be configured for the DMA transfer.
	aDeviceDrcmr:  Specifies the device to be mapped to the DMA channel for the transfer
			  	   operation.
	aLignment: Indicated whether to enable byte alignment for memory accesses			  
			  
	                 
  Output Arguments:
  	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacCfgChannelNoDescTransfer(
						  			 XLLP_UINT32_T             aSourceAddr,  	
						  			 XLLP_UINT32_T             aTargetAddr,  	
							    	 XLLP_DMAC_COMMAND_T*      pCmd,
						  			 XLLP_DMAC_CHANNEL_T       aChannel,
								     XLLP_DMAC_DEVICE_T        aDeviceDrcmr,						  			 
						  			 XLLP_DMAC_ALIGNMENT_T     aLignment
					     			 )
{
	XLLP_VUINT32_T aCommand, aTargetData;
	
	if (aLignment == XLLP_DMAC_ALIGNMENT_ON)
	{
		aTargetData = pDmacHandle->DALGN;
		aTargetData |= (1 << aChannel);
		pDmacHandle->DALGN = aTargetData;
	}

	/* Map device to channel                 */
	/* No mapping required for mem2mem moves */
	if (aDeviceDrcmr != XLLP_DMAC_MEM2MEM_MOVE)
	{
		XllpDmacMapDeviceToChannel(aDeviceDrcmr, aChannel);
	}	

	/* Set the No Descriptor Fetch bit in DCSR*/
	aTargetData  = pDmacHandle->DCSR[aChannel];
	aTargetData |= XLLP_DMAC_DCSR_NO_DESC_FETCH;
	pDmacHandle->DCSR[aChannel]  =	aTargetData;


	/* Construct command value from pCmd      */
	aCommand = (
				(pCmd->aLen) | ((pCmd->aWidth) << XLLP_BIT_FIELD_14)|
				((pCmd->aSize) << XLLP_BIT_FIELD_16)      |
				((pCmd->aEndian) << XLLP_BIT_FIELD_18)    | 
				((pCmd->aFlyByT) << XLLP_BIT_FIELD_19)    |
				((pCmd->aFlyByS) << XLLP_BIT_FIELD_20)    |
				((pCmd->aEndIrqEn)   << XLLP_BIT_FIELD_21)|
				((pCmd->aStartIrqEn) << XLLP_BIT_FIELD_22)|
				((pCmd->aAddrMode)   << XLLP_BIT_FIELD_23)|
				((pCmd->aCmpEn)   << XLLP_BIT_FIELD_25)   |
				((pCmd->aFlowTrg) << XLLP_BIT_FIELD_28)   |
				((pCmd->aFlowSrc) << XLLP_BIT_FIELD_29)   |
				((pCmd->aIncTrgAddr) << XLLP_BIT_FIELD_30)|
				((pCmd->aIncSrcAddr) << XLLP_BIT_FIELD_31)
			   );	   	

	pDmacHandle->DDG[aChannel].DSADR  =	aSourceAddr;
	pDmacHandle->DDG[aChannel].DTADR  =	aTargetAddr;
	pDmacHandle->DDG[aChannel].DCMD   =	aCommand;
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacStartTransfer

  Description: XllpDmacStartTransfer starts the DMA transfer process for the 
  			   specified channel.
  			   
  Global Registers Modified: 
	DCSRx register, where x indicate the channel number
  
  Input Arguments:
	aChannel:  Specifies the channel to start the DMA transfer on.	                 
  
  Output Arguments:
  	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacStartTransfer( XLLP_DMAC_CHANNEL_T aChannel )
{
	pDmacHandle->DCSR[aChannel] |= XLLP_DMAC_DCSR_RUN;
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacStopTransfer

  Description: XllpDmacStopTransfer stops the DMA transfer process for the 
  			   specified channel.
  			   
  Global Registers Modified: 
	DCSRx register, where x indicate the channel number
  
  Input Arguments:
	aChannel:  Specifies the channel to stop the DMA transfer on.  
  
  Output Arguments:
  	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/

void XllpDmacStopTransfer( XLLP_DMAC_CHANNEL_T aChannel )
{
	pDmacHandle->DCSR[aChannel]  &= ~XLLP_DMAC_DCSR_RUN;
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacAttachDesc

  Description: XllpDmacAttachDesc attaches the specified descriptor to the 
  			   chained/linked descriptors.  Users who want to attach a descriptor
  			   on the fly can use this function.  Note that if DMA transfers are
  			   already in session for the specified channel, this transfer will 
  			   be stopped, attach the descriptor, and then restarted.  Users can
  			   use XllpDmacGetChannelStatus to request status of the channel, 
  			   ensure that the current transfer completes before attaching 
  			   additional descriptors.
  			   
  Global Registers Modified: 
	DCSRx, and DDADRx registers, where x indicate the channel number  
  
  Input Arguments:
	pDesc:  Holds the pointer to the descriptor to be attached.  The pointer 
			cannot be null.
	pPrevDesc:  Holds the pointer to the descriptor in the chain that is to have
				the "to be attached" descriptor pDesc as the next descriptor.  
				That is, pPrevDesc will have pDesc as the next descriptor, and 
				pDesc will have pNextDesc as its next descriptor in the chain.  
				pPrevDesc points to pDesc and pDesc points to pNextDesc.  
	pNextDesc:  Holds the pointer to the descriptor in the chain that will be 
			    the next descriptor to pDesc. See description of pPrevDesc for 
			    more details.
	pStartDesc: Descriptor to re-start DMA transfer from			
	aChannel:  Specifies the DMA channel the linked descriptors are utilizing 
			   for the transfer process.
  
  Output Arguments:
  	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacAttachDesc(
			    		P_XLLP_DMAC_DESCRIPTOR_T  pDesc,
			    		P_XLLP_DMAC_DESCRIPTOR_T  pPrevDesc,
			    		P_XLLP_DMAC_DESCRIPTOR_T  pNextDesc,
			    		P_XLLP_DMAC_DESCRIPTOR_T  pStartDesc,
						XLLP_DMAC_CHANNEL_T       aChannel
					   )
{
	XLLP_VUINT32_T aTargetValue; 
	XLLP_BOOL_T    aChannelRunStatus = XLLP_FALSE;

	aTargetValue  = pDmacHandle->DCSR[aChannel];
	if((aTargetValue & (XLLP_UINT32_T)XLLP_DMAC_DCSR_RUN) == (XLLP_UINT32_T)XLLP_DMAC_DCSR_RUN)
	{
		aTargetValue &= ~XLLP_DMAC_DCSR_RUN;
		pDmacHandle->DCSR[aChannel] = aTargetValue;
	  	aChannelRunStatus = XLLP_TRUE;  
	}

	/* Attach the descriptor             */
	pPrevDesc->DDADR = (XLLP_UINT32_T)pDesc;
	pDesc->DDADR     = (XLLP_UINT32_T)pNextDesc;
	pDmacHandle->DDG[aChannel].DDADR = (XLLP_UINT32_T)pStartDesc;

	/* If channel was running, restart it*/
	if (aChannelRunStatus == XLLP_TRUE)
	{
		aTargetValue |= XLLP_DMAC_DCSR_RUN;
		pDmacHandle->DCSR[aChannel] = aTargetValue;
	}
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacDetachDesc

  Description: XllpDmacDetachDesc detaches the specified descriptor to the 
  			   chained/linked descriptors.  Users who want to detach a 
  			   descriptor on the fly can use this function.  Note that if 
  			   DMA transfers are already in session for the specified channel,
  			   this transfer will be stopped, detach the descriptor, and then
  			   restarted.  Users can use XllpDmacGetChannelStatus to request 
  			   status of the channel, ensure that the current transfer completes
  			   before detaching descriptors. 
  			   
  Global Registers Modified: 
	DCSRx, and DDADRx registers, where x indicate the channel number  
  
  Input Arguments:
	pDesc:  Holds the pointer to the descriptor to be detached.  The pointer
			cannot be null.
	pPrevDesc:  Holds the pointer to the descriptor in the chain that had the 
				"to be detached" descriptor pDesc as the next descriptor.  That
				is, pPrevDesc had pDesc as the next descriptor, but will now 
				have pNextDesc as its next descriptor in the chain.  Before 
				executing the primitive pPrevDesc pointed to pDesc and pDesc 
				pointed to pNextDesc, after executing this function pPrevDesc 
				will be pointer to pNextDesc.  Neither pointers can have a null.
	pNextDesc:  Holds the pointer to the descriptor in the chain that will be 
				the next descriptor to pPrevDesc. See description of pPrevDesc 
				for more details.  The pointer cannot be null
	pStartDesc: Descriptor to re-start DMA transfer from			
	aChannel:  Specifies the DMA channel this chained descriptors are utilizing 
			   for the transfer process.  Valid values are 0-31.
  
  Output Arguments:
  	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacDetachDesc(
			    		P_XLLP_DMAC_DESCRIPTOR_T  pDesc,
			    		P_XLLP_DMAC_DESCRIPTOR_T  pPrevDesc,
			    		P_XLLP_DMAC_DESCRIPTOR_T  pNextDesc,
			    		P_XLLP_DMAC_DESCRIPTOR_T  pStartDesc,
						XLLP_DMAC_CHANNEL_T       aChannel
					   )
{
	XLLP_VUINT32_T aTargetValue; 
	XLLP_BOOL_T    aChannelRunStatus = XLLP_FALSE;

	/* Read Control & Status register,   */
	/* verify if transfer is in process. */
	/* If so, stop the channel           */
	aTargetValue  = pDmacHandle->DCSR[aChannel];
	if((aTargetValue & (XLLP_UINT32_T)XLLP_DMAC_DCSR_RUN) == (XLLP_UINT32_T)XLLP_DMAC_DCSR_RUN)
	{
		aTargetValue &= ~XLLP_DMAC_DCSR_RUN;
		pDmacHandle->DCSR[aChannel] = aTargetValue;
	  	aChannelRunStatus = XLLP_TRUE;  
	}

	/* Detach descriptor                 */
	pPrevDesc->DDADR = (XLLP_UINT32_T)pNextDesc;

	pDmacHandle->DDG[aChannel].DDADR = (XLLP_UINT32_T)pStartDesc;

	/* If channel was running, restart it*/
	if (aChannelRunStatus == XLLP_TRUE)
	{
		aTargetValue |= XLLP_DMAC_DCSR_RUN;
		pDmacHandle->DCSR[aChannel] = aTargetValue;
	}
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacGetChannelStatus

  Description: XllpDmacGetChannelStatus returns the status of the DMA transfer 
  			   process for the specified channel.  The function can return a 
  			   status or multiple status for a particular DMA channel.
  			   
  Global Registers Modified: 
	None
		  
  Input Arguments:
	aChannel:  Specifies the DMA channel whose status to return.  Valid values 
			   are 0-31.
	aStatus:  Specifies the DMA channel status the caller is requesting.  aStatus
			  could be an element of type XLLP_DMAC_CHANNEL_STATUS_T or multiple 
			  elements of the said type.
  
  Output Arguments:
	pStatus: Holds the DMA channel status requested by the caller.  pStatus may
			 contain one or multiple statuses depending on aStatus.  
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacGetChannelStatus(
							 XLLP_DMAC_CHANNEL_T          aChannel,
							 XLLP_DMAC_CHANNEL_STATUS_T   aStatus,
							 P_XLLP_DMAC_CHANNEL_STATUS_T pStatus
							 )
{
 /* IMPLEMENTION IS EXCLUDED BECAUSE IT IS A WRAPPER FUNCTION   */

 /* WILL BE ADDED IF THERE ARE USERS WHO WILL LIKE TO USE IT*/
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacClearChannelStatus

  Description: XllpDmacClearChannelStatus clears the status of the DMA transfer 
  			   process for the specified channel.  The function can clear one or
  			   multiple statuses for a particular DMA channel.
  			   
  Global Registers Modified: 
	DCSRx register, where x denotes the channel number.		  
  
  Input Arguments:
	aChannel:  Specifies the DMA channel whose status to clear.  Valid values 
			   are 0-31.
	aStatus:  Specifies the DMA channel status(s) to clear.  aStatus could be an
	 		  element of type XLLP_DMAC_CHANNEL_STATUS_T or multiple elements of
	 		  the said type.
  
  Output Arguments:
	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacClearChannelStatus(
							   XLLP_DMAC_CHANNEL_T        aChannel,
							   XLLP_DMAC_CHANNEL_STATUS_T aStatus 
					           )
{
 /* IMPLEMENTION IS EXCLUDED BECAUSE IT IS A WRAPPER FUNCTION   */

 /* WILL BE ADDED IF THERE ARE USERS WHO WILL LIKE TO USE IT*/
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacMapDeviceToChannel

  Description: XllpDmacMapDeviceToChannel is used to map the specified device
  			   to the specified DMA channel. 
  			   
  Global Registers Modified: 
	DRCMRx register, where x denotes the channel number.  
  
  Input Arguments:
	aDeviceDrcmr:  Specifies the device to be mapped to the DMA channel.  Valid 
			  parameters are values of type  XLLP_DMAC_DRCMR_T, otherwise
			  invalid.
	aChannel: Specifies the DMA channel to map the specified device to.  Valid 
			  channel values range from 0-31.
  
  Output Arguments:
	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacMapDeviceToChannel(
							   XLLP_DMAC_DRCMR_T   aDeviceDrcmr,
							   XLLP_DMAC_CHANNEL_T aChannel
						       )
{
	if (aDeviceDrcmr < XLLP_DMAC_DRCMR1_NUM)
	{
		/* Enable device-to-channel mapping */
		pDmacHandle->DRCMR1[aDeviceDrcmr] = (XLLP_DMAC_DRCMR_ENABLE + aChannel);
	}
	else
	{
		/* Enable device-to-channel mapping */
		pDmacHandle->DRCMR2[aDeviceDrcmr-XLLP_DMAC_DRCMR1_NUM] = (XLLP_DMAC_DRCMR_ENABLE + aChannel);
	}	
}

/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacUnMapDeviceToChannel

  Description: XllpDmacUnMapDeviceToChannel is used to unmap the specified 
  			   device from the specified DMA channel.  
  			   
  Global Registers Modified: 
	DRCMRx register, where x denotes the channel number.  
  
  Input Arguments:
	aDeviceDrcmr:  Specifies the device to be unmapped from the DMA channel.  Valid
			  parameters are values of type XLLP_DMAC_DRCMR_T, otherwise invalid.
	aChannel: Specifies the DMA channel to unmap the specified device from.  
			  Valid channel values range from 0-31.

  Output Arguments:
	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacUnMapDeviceToChannel(
								 XLLP_DMAC_DRCMR_T   aDeviceDrcmr,
								 XLLP_DMAC_CHANNEL_T aChannel
								 )
{
	if (aDeviceDrcmr < XLLP_DMAC_DRCMR1_NUM)
	{
		/* Disable device-to-channel mapping */
		pDmacHandle->DRCMR1[aDeviceDrcmr] = (XLLP_DMAC_DRCMR_DISABLE + aChannel);
	}
	else
	{
		/* Disable device-to-channel mapping */
		pDmacHandle->DRCMR2[aDeviceDrcmr-XLLP_DMAC_DRCMR1_NUM] = (XLLP_DMAC_DRCMR_DISABLE + aChannel);
	}	
}


/******************************************************************************
XLLP_DOC_HDR_BEGIN

  Function Name: XllpDmacSetEor

  Description: XllpDmacSetEor sets the stop on EOR functionality for the specified channel
  			   
  Global Registers Modified: 
	DCSRx register, where x indicate the channel number
  
  Input Arguments:
	aChannel:  Specifies the channel for which to set the stop on EOR functionality   
  
  Output Arguments:
  	None
  Return Value: 
	None 

XLLP_DOC_HDR_END
*******************************************************************************/
void XllpDmacSetEor( XLLP_DMAC_CHANNEL_T aChannel )
{
	XLLP_VUINT32_T aTargetData;     

	aTargetData  = pDmacHandle->DCSR[aChannel];	
    aTargetData |= XLLP_DMAC_DCSR_EOR_STOP_EN;	
	pDmacHandle->DCSR[aChannel]  = aTargetData;
}
