//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
/*++
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Module Name:  

Abstract:

    Platform dependent PCMCIA initialization functions

Notes: 
--*/
#include <windows.h>
#include <types.h>
#include <socksv2.h>
#include <memory.h>
#include <ceddk.h>
#include <nkintr.h>
#include "pdsocket.h"


//////////////////////////////////////////////////////////////////////////

// special registry values for this driver
#define IRQ_VALUE_NAME           TEXT("Irq")
#define SYSINTR_VALUE_NAME       TEXT("SysIntr")
#define POLLEDDEVICES_VALUE_NAME TEXT("PolledDevices")
#define CLIENT_IRQ_VALUE_NAME    TEXT("ClientIrq")
#define POLL_TIMEOUT_NAME        TEXT("PollTimeout")
#define POLLING_MODE_NAME        TEXT("PollingMode")
#define SLOT_NUMBER_NAME         TEXT("SlotNumber")
#define DISABLE_SOCKET_NAME      TEXT("DisableSocket")
#define CSC_THREAD_PRIORITY_NAME TEXT("CSCThreadPriority")
#define POWER_OPTION_NAME        TEXT("RegPowerOption")

#define DEFAULT_PRIORITY         101

const SS_POWER_ENTRY CPcmciaCardSocket::m_rgPowerEntries[NUM_POWER_ENTRIES] =
{
    { 0,    PWR_SUPPLY_VCC  | PWR_SUPPLY_VPP1 | PWR_SUPPLY_VPP2 },
    { 33,   PWR_SUPPLY_VCC  | PWR_SUPPLY_VPP1 | PWR_SUPPLY_VPP2 },
    { 50,   PWR_SUPPLY_VCC  | PWR_SUPPLY_VPP1 | PWR_SUPPLY_VPP2 }
};

DWORD CPcmciaCardSocket::m_dwSocketLastIndex = 1;


STATUS CPcmciaCardSocket::GetPowerEntry( PDWORD pdwNumOfEntry,
                                         PSS_POWER_ENTRY pPowerEntry )
{
    STATUS status = CERR_BAD_ARGS;
    if( pdwNumOfEntry != NULL && pPowerEntry != NULL )
    {
        DWORD dwNumOfCopied = min( *pdwNumOfEntry, NUM_POWER_ENTRIES );
        if( dwNumOfCopied != 0 )
        {
            memcpy( pPowerEntry,
                    m_rgPowerEntries,
                    dwNumOfCopied * sizeof( SS_POWER_ENTRY ) );
            *pdwNumOfEntry = dwNumOfCopied;
            status = CERR_SUCCESS;
        }
    }
    return status;
}


CPcmciaBusBridge::CPcmciaBusBridge( LPCTSTR RegPath ) : CPCCardBusBridgeBase( RegPath ),
                                                        CMiniThread( 0,
                                                                     TRUE )
{

    //m_fPCICCritSecInitialized = false;
    m_hISTEvent = NULL;
    m_prgCardSocket = NULL;
    m_rgfPowerCycleEvent = FALSE;
    m_rgForceEject = FALSE;
    m_fPollingMode = TRUE;
};


CPcmciaBusBridge::~CPcmciaBusBridge()
{
    GetSocketNumberFromCS( FALSE );

    // Terminate IST
    m_bTerminated = TRUE;
    if( m_hISTEvent )
    {
        SetEvent( m_hISTEvent );
        ThreadTerminated( 1000 );
        if( !m_fPollingMode )
        {
            InterruptDisable( m_dwCSCSysIntr );
        }
        CloseHandle( m_hISTEvent );
    };

    RemovePcmciaCardSocket();

    //if( m_fPCICCritSecInitialized )
    //{
        //DeleteCriticalSection( &m_PddSocketCrit );
        //DeleteCriticalSection( &m_PddWindowCrit );
    //}

    UnMapDeviceRegisters();

}


BOOL CPcmciaBusBridge::InstallIsr()
{

    // CreateIST Event.
    m_hISTEvent = CreateEvent( 0, FALSE, FALSE, NULL );

    if( !m_fPollingMode )
    {
        InterruptInitialize( m_dwCSCSysIntr, m_hISTEvent, 0, 0 );
        //InterruptInitialize( m_dwCSCSysIntr, m_hISTEvent, 0, 0 );
    }

    return(TRUE);
}


BOOL CPcmciaBusBridge::InitializeBridgeHW( void )
{
    // TODO - handle initializing only the current slot...

    // Configure the MECR register to inform the memory controller about the
    // total number of sockets on Mainstone (0x1 => 2 sockets).
    //
    m_pMemCtrlRegs->mecr   = 0x00000001;

    m_strPCCardSocketHandle.pstrGpioRegsHandle    = (XLLP_GPIO_T *) m_pGPIORegs;
//    m_strPCCardSocketHandle.pstrBcrHandle         = (XLLP_BCR_T *)  m_pBLRegs;
    m_strPCCardSocketHandle.pstrOstRegsHandle     = (XLLP_OST_T *)  m_pOSTRegs;
    m_strPCCardSocketHandle.pstrMemCtrlRegsHandle = (XLLP_MEMORY_CONTROL_REGISTER_T *) m_pMemCtrlRegs;

    m_strPCCardSocketHandle.pstrPCCardSocketState = &m_strPCCardSocketState;
    m_strPCCardSocketHandle.pstrCardTimingHandle  = (XLLP_CARDTIMING_T *)&m_strPCCardExpMemTiming;

    m_pstrPCCardSocketHandle = &m_strPCCardSocketHandle;       

    // Set up memory controller timings for both sockets.
    //
    XllpPCCardSetExpMemTiming(m_pstrPCCardSocketHandle);

    // Configure the GPIO signals for both sockets.
    //
//    XllpPCCardConfigureGPIOs(m_pstrPCCardSocketHandle);	
	XllpPCCardHWSetup(m_pstrPCCardSocketHandle);

//    m_pBLRegs->pcmcia0_srcr  = 0;
//    m_pBLRegs->pcmcia1_srcr  = 0;

//    m_pBLRegs->pcmcia0_srcr |= (XLLP_BIT_3);
//    m_pBLRegs->pcmcia1_srcr |= (XLLP_BIT_3);

    return(TRUE);
}


BOOL CPcmciaBusBridge::UnMapDeviceRegisters( void )
{

/*	if(m_pBLRegs)
	{
		VirtualFree((PVOID)m_pBLRegs, 0, MEM_RELEASE);
		m_pBLRegs = NULL;
	}*/
	if(m_pGPIORegs)
	{
		VirtualFree((PVOID)m_pGPIORegs, 0, MEM_RELEASE);
		m_pGPIORegs = NULL;
	}
	if(m_pMemCtrlRegs)
	{
		VirtualFree((PVOID)m_pMemCtrlRegs, 0, MEM_RELEASE);
		m_pMemCtrlRegs = NULL;
	}
	if(m_pOSTRegs)
	{
		VirtualFree((PVOID)m_pOSTRegs, 0, MEM_RELEASE);
		m_pOSTRegs = NULL;
	}

    return(TRUE);
}


BOOL CPcmciaBusBridge::MapDeviceRegisters( void )
{
    PHYSICAL_ADDRESS PA;

//    PA.QuadPart = MAINSTONEII_BASE_REG_PA_FPGA;
//    m_pBLRegs = (volatile MAINSTONEII_BLR_REGS *) MmMapIoSpace(PA, sizeof(MAINSTONEII_BLR_REGS), FALSE);

    PA.QuadPart = BULVERDE_BASE_REG_PA_GPIO;
    m_pGPIORegs = (volatile BULVERDE_GPIO_REG *) MmMapIoSpace(PA, sizeof(BULVERDE_GPIO_REG), FALSE);

    PA.QuadPart = BULVERDE_BASE_REG_PA_MEMC;
    m_pMemCtrlRegs = (volatile BULVERDE_MEMCTRL_REG *) MmMapIoSpace(PA, sizeof(BULVERDE_MEMCTRL_REG), FALSE);

    PA.QuadPart = BULVERDE_BASE_REG_PA_OST;
    m_pOSTRegs = (volatile BULVERDE_OST_REG *) MmMapIoSpace(PA, sizeof(BULVERDE_OST_REG), FALSE);

    if (/*!m_pBLRegs ||*/ !m_pGPIORegs || !m_pMemCtrlRegs || !m_pOSTRegs)
    {
        return(FALSE);
    }

    return(TRUE);

}   // MapDeviceRegisters.


//
// Function to get the initial settings from the registry
//
// NOTE: lpRegPath is assumed to be under HKEY_LOCAL_MACHINE
//
// Returns ERROR_SUCCESS on success or a Win32 error code on failure
//
DWORD CPcmciaBusBridge::GetRegistryConfig( void )
{
    DWORD dwRet = 1;
    DWORD dwSize, dwType, dwData;

    // Get the PCMCIA window configurations
    if( !LoadWindowsSettings() )
    {
        dwRet = ERROR_INVALID_DATA;
        goto grc_fail;
    }

    // Read the registry to determine the physical slot number we're dealing with.
    dwSize = sizeof( DWORD );
    if( !RegQueryValueEx( SLOT_NUMBER_NAME,
                          &dwType,
                          ( PUCHAR ) & dwData,
                          &dwSize ) )
    {
        DEBUGMSG( ZONE_PDD,
                  ( TEXT( "PCMCIA:GetRegistyConfig RegQueryValueEx(%s) failed\r\n" ),
                    SLOT_NUMBER_NAME ) );
        goto grc_fail;
    }
    m_SlotNumber = (UINT8)dwData;

    // Get the polling mode value.
    dwSize = sizeof( DWORD );
    if( !RegQueryValueEx( POLLING_MODE_NAME,
                          &dwType,
                          ( PUCHAR ) & dwData,
                          &dwSize ) )
    {
        m_fPollingMode = TRUE; // RegQueryValueEx failed, default to TRUE
    }
    else
    {
        m_fPollingMode = dwData ? TRUE : FALSE;
    }

    // Get client (function) IRQ.
    dwSize = sizeof( DWORD );
    if( !RegQueryValueEx( CLIENT_IRQ_VALUE_NAME,
                          &dwType,
                          ( PUCHAR ) & dwData,
                          &dwSize ) )
    {
        DEBUGMSG( ZONE_PDD,
                  ( TEXT( "PCMCIA:GetRegistyConfig RegQueryValueEx(%s) failed\r\n" ),
                    CLIENT_IRQ_VALUE_NAME ) );
        goto grc_fail;
    }
    m_dwClientIrq = dwData;

// TODO - get CD IRQ and SYSINTR - no polling.

    // Get card status change (CSC) IRQ and SYSINTR values.
    // NOTE: the CSC and CD IRQ values are part of the same IRQ multi-string in the registry.
    dwSize = sizeof( DWORD );
    if( !RegQueryValueEx( IRQ_VALUE_NAME,
                          &dwType,
                          ( PUCHAR ) & dwData,
                          &dwSize ) )
    {
        m_dwCSCIrq = 0;
        m_fPollingMode = FALSE;
    }
    else
    {
        m_dwCSCIrq = dwData;
    }
    dwSize = sizeof( DWORD );
    if( !RegQueryValueEx( SYSINTR_VALUE_NAME,
                          &dwType,
                          ( PUCHAR ) & dwData,
                          &dwSize ) )
    {
        m_dwCSCSysIntr = 0;
        m_dwCSCIrq = 0;
        m_fPollingMode = FALSE;
    }
    else
    {
        m_dwCSCSysIntr = dwData;
    }

    // Get the polling timeout value.
    dwSize = sizeof( DWORD );
    if( !RegQueryValueEx( POLL_TIMEOUT_NAME,
                          &dwType,
                          ( PUCHAR ) & dwData,
                          &dwSize ) )
    {
        // RegQueryValueEx failed; if pooling, set the timeout to 0.5 sec, otherwise set to INFINITE
        m_dwPollTimeout = m_fPollingMode ? 500 : INFINITE;
    }
    else
    {
        m_dwPollTimeout = dwData;
    }

    // Get thread priority from registry and set it.
    m_uPriority = DEFAULT_PRIORITY;
    dwSize = sizeof( DWORD );
    if( RegQueryValueEx( CSC_THREAD_PRIORITY_NAME,
                         &dwType,
                         ( PUCHAR ) & dwData,
                         &dwSize ) )
    {
        m_uPriority = ( UINT )dwData;
    }

    dwRet = ERROR_SUCCESS;

    grc_fail : return dwRet;
}   // GetRegistryConfig


BOOL CPcmciaBusBridge::InitCardBusBridge( void )
{

    // Get PCCARD controller registry information.
    if((GetRegistryConfig()) != ERROR_SUCCESS)
    {
        DEBUGMSG( ZONE_PDD,
                  ( TEXT( "PDCardInitServices GetRegistryConfig failed.\r\n" )));
        return(FALSE);
    }

    // Map PCCARD controller device buffers.
    if( !MapDeviceRegisters() )
    {
        DEBUGMSG( ZONE_PDD,
                  ( TEXT( "PDCardInitServices MapDeviceRegisters failed %d\r\n" )));
        return(FALSE);
    }

    // Initialize the PCMCIA bridge hardware.
    if( !InitializeBridgeHW() )
    {
        DEBUGMSG( ZONE_PDD,
                  ( TEXT( "PDCardInitServices InitializeBridgeHW failed %d\r\n" )));
        return(FALSE);
    }

    // Initialize slot interrupts.
    InstallIsr();

    return(TRUE);
}


BOOL CPcmciaBusBridge::Init()
{
    if(!loadPcCardEntry())
    {
        return(FALSE);
    }

    if(!InitCardBusBridge())
    {
        return(FALSE);
    }

    if (!GetSocketNumberFromCS( TRUE ))
    {
        return(FALSE);
    }

    // Set Tread Priority.
    CeSetPriority( m_uPriority );
    m_bTerminated = FALSE;

    ThreadStart();
    return(TRUE);
}


BOOL CPcmciaBusBridge::GetRegPowerOption( PDWORD pOption )
{
    if( pOption )
    {
        return GetRegValue( POWER_OPTION_NAME,
                            ( PBYTE ) pOption,
                            sizeof( DWORD ) );
    }
    return TRUE;
}


void CPcmciaBusBridge::PowerMgr( BOOL bPowerDown )
{
}


BOOL CPcmciaBusBridge::NeedPowerResuming()
{
    m_bResumeFlag = TRUE;
    SetInterruptEvent( m_dwCSCSysIntr );
    return TRUE;
}


void CPcmciaBusBridge::PowerCycleEvent(UINT16 nSlotNumber)
{
    Lock();
    m_rgfPowerCycleEvent = TRUE;
    SetEvent( m_hISTEvent );
    Unlock();
}
void  CPcmciaBusBridge::ForceEjectEvent(UINT16 nSlotNumber)
{
    Lock();
    m_rgForceEject = TRUE;
    SetEvent( m_hISTEvent );
    Unlock();
}


DWORD CPcmciaBusBridge::ThreadRun() // This is card status change IST
{
    UINT16 statusReg            = 0;
    BOOL  bLastCardDetectStatus = FALSE;
    BOOL  bLastIRQReadyStatus   = FALSE;
    BOOL  bLastBvd1Status       = FALSE;
    BOOL  bLastBvd2Status       = FALSE;
    DWORD dwLastNotifyEvents    = FALSE;
    DWORD  dwNotifyEvents       = 0;
    BOOL bIsCardInserted        = FALSE;

	statusReg = XllpPCCardSocketState( m_pstrPCCardSocketHandle, m_SlotNumber);

    // Test if a PCMCIA card is already inserted in one of the sockets
    switch(m_SlotNumber)
    {
    case 1:
//        statusReg             = m_pBLRegs->pcmcia1_srcr;
        bLastCardDetectStatus = !((statusReg & PCMCIA_S1_nCD_MASK) >> 5);
        if (bLastCardDetectStatus)
        {
            bLastIRQReadyStatus   = ((statusReg & PCMCIA_S1_nIRQ_MASK) >> 10);
            bLastBvd1Status       = !((statusReg & PCMCIA_S1_nSTSCHG_BVD1_MASK) >> 8);
            bLastBvd2Status       = !((statusReg & PCMCIA_S1_nSPKR_BVD2_MASK) >> 9);
        }
        break;
    case 0:
    default:
//        statusReg             = m_pBLRegs->pcmcia0_srcr;
        bLastCardDetectStatus = !((statusReg & PCMCIA_S0_nCD_MASK) >> 5);
        if (bLastCardDetectStatus)
        {
            bLastIRQReadyStatus   = ((statusReg & PCMCIA_S0_nIRQ_MASK) >> 10);
            bLastBvd1Status       = !((statusReg & PCMCIA_S0_nSTSCHG_BVD1_MASK) >> 8);
            bLastBvd2Status       = !((statusReg & PCMCIA_S0_nSPKR_BVD2_MASK) >> 9);
        }
        break;
    }

    if (bLastCardDetectStatus)
    {
        InsertPcmciaCardSocket(CreatePCMCIASocket(this, m_SlotNumber));
        bIsCardInserted = true;
    }

    while( !m_bTerminated )
    {
        BOOL bInterrupt;

        ASSERT(m_hISTEvent != NULL);

        bInterrupt = (WaitForSingleObject(m_hISTEvent, m_dwPollTimeout) != WAIT_TIMEOUT);
        if( m_fPollingMode )    // Fake the interrupt if we're polling.
        {
            bInterrupt = true;
        }

        if (!bInterrupt)    // we have reached a timeout in non-pooling mode or something bad has occured.
        {
            continue;
        }

        Lock();

        //
        // Determine the slot status.
        //
        BOOL bCardDetectStatus;
        BOOL bIRQReadyStatus;
        BOOL bBvd1Status;
        BOOL bBvd2Status;

		statusReg = XllpPCCardSocketState( m_pstrPCCardSocketHandle, m_SlotNumber);

        switch(m_SlotNumber)
        {
        case 1:
//            statusReg         = m_pBLRegs->pcmcia1_srcr;
            bCardDetectStatus = !((statusReg & PCMCIA_S1_nCD_MASK) >> 5);
            bBvd1Status       = !((statusReg & PCMCIA_S1_nSTSCHG_BVD1_MASK) >> 8);
            bBvd2Status       = !((statusReg & PCMCIA_S1_nSPKR_BVD2_MASK) >> 9);
            bIRQReadyStatus   = ((statusReg & PCMCIA_S1_nIRQ_MASK) >> 10);
            break;
        case 0:
        default:
//            statusReg         = m_pBLRegs->pcmcia0_srcr;
            bCardDetectStatus = !((statusReg & PCMCIA_S0_nCD_MASK) >> 5);
            bBvd1Status       = !((statusReg & PCMCIA_S0_nSTSCHG_BVD1_MASK) >> 8);
            bBvd2Status       = !((statusReg & PCMCIA_S0_nSPKR_BVD2_MASK) >> 9);
            bIRQReadyStatus   = ((statusReg & PCMCIA_S0_nIRQ_MASK) >> 10);
            break;
        }

        // if nothing has changed, continue to the next slot
        if( !m_rgfPowerCycleEvent && !m_rgForceEject &&
            bCardDetectStatus == bLastCardDetectStatus &&
            bBvd1Status == bLastBvd1Status &&
            bBvd2Status == bLastBvd2Status &&
            bIRQReadyStatus == bLastIRQReadyStatus)
        {
            Unlock();
            continue;
        }

        bLastCardDetectStatus = bCardDetectStatus;
        bLastBvd1Status       = bBvd1Status;
        bLastBvd2Status       = bBvd2Status;
        bLastIRQReadyStatus   = bIRQReadyStatus;


        //
        // Figure out the socket state
        //

        if(bCardDetectStatus)
        {
            dwNotifyEvents |= SOCK_EVENT_CD;

            if (bIRQReadyStatus)
            {
                dwNotifyEvents |= SOCK_EVENT_READY;
            }

            if(bBvd1Status)
            {
                dwNotifyEvents |= SOCK_EVENT_BVD1;
            }
    
            if(bBvd2Status)
            {
                dwNotifyEvents |= SOCK_EVENT_BVD2;
            }
						
        }
        else
        {
            dwNotifyEvents &= ~(SOCK_EVENT_CD);	               
        }

        Unlock();
        
        if (m_rgForceEject) {
            m_rgForceEject = FALSE;
            RemovePcmciaCardSocket(); 
            bIsCardInserted = false;
        }
        
        if( bIsCardInserted && !bCardDetectStatus )
        {
            RemovePcmciaCardSocket();
            bIsCardInserted = false;
        }
        else if ( !bIsCardInserted && bCardDetectStatus )
        {
            RemovePcmciaCardSocket();
            InsertPcmciaCardSocket(CreatePCMCIASocket(this, m_SlotNumber));
            bIsCardInserted = true;
        }
        else
        {

            // Other Event left for type specific socket
            DWORD dwNotifyEventsChanged = dwLastNotifyEvents ^ dwNotifyEvents;
            dwLastNotifyEvents = dwNotifyEvents;
            if (m_prgCardSocket)
            {
                m_prgCardSocket->SocketEventHandle( dwNotifyEventsChanged, dwNotifyEvents );
            }
        }

        if( !m_fPollingMode )
        {
            DEBUGMSG( ZONE_FUNCTION,
                      ( L"CardBus: PCCardBus Call InterruptDone\r\n" ) );
            InterruptDone( m_dwCSCSysIntr );
        }

    } // while(!m_bTerminated) 

    return(0);

}


BOOL CPcmciaBusBridge::SetupWakeupSource( BOOL bSet )
{
    return(FALSE);
}


void CPcmciaBusBridge::CallBackToCardService( UINT16 nSocketNumber,
                                              HANDLE hSocket,
                                              PSS_SOCKET_STATE pSocketState )
{
    if( m_pCallBackToCardService )
    {
        __try
        {
            m_pCallBackToCardService( hSocket,
                                      nSocketNumber,
                                      pSocketState );
        } __except( EXCEPTION_EXECUTE_HANDLER )
        {
            DEBUGCHK( FALSE );
        }
    }
    else
    {
        DEBUGCHK( FALSE );
    }
}


BOOL CPcmciaBusBridge::GetSocketNumberFromCS(BOOL bGet)
{
    STATUS status = CERR_BAD_ARGS;
    if (m_pRequestSocketNumber && m_pDeleteSocket)
    {
        __try
        {
            status = (bGet ?
                      m_pRequestSocketNumber(&m_SocketNumber,
                                             sizeof( MAINSTONEIISocketServiceStatic ),
                                             &MAINSTONEIISocketServiceStatic,
                                             GetSocketName()) :
                       m_pDeleteSocket(m_SocketNumber));
        } __except(EXCEPTION_EXECUTE_HANDLER)
        {
            DEBUGCHK(FALSE);
            status = CERR_BAD_ARGS;
        }
    }
    DEBUGCHK(status == CERR_SUCCESS);
    return (status == CERR_SUCCESS);
}


