//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
/* Copyright 1999,2000,2001 Intel Corp.  */
/*++

Module Name:  tchpdd.c

Abstract:
 Intel XSC1 StrongARM  platform touch driver routines.

Notes:
 Created: July 22, 1997
 Modified January 17, 2002

--*/
//------------------------------------------------------------------------------
// Include Files
//------------------------------------------------------------------------------
#include <windows.h>
#include <nkintr.h>
#include <tchddsi.h>
#include <bsp.h>
#include "touch.h"
#include "aclink.h"

//------------------------------------------------------------------------------
// Defines
//------------------------------------------------------------------------------
#define REV_2A    0x2a0
#define REV_1B    0x2a

//------------------------------------------------------------------------------
// External Variables
//------------------------------------------------------------------------------
extern "C" const int MIN_CAL_COUNT = 25;

//------------------------------------------------------------------------------
// Local Variables
//------------------------------------------------------------------------------
static EXPECTED_INT_EVENT g_NextExpectedInterrupt = PEN_DOWN;
static UINT32 g_ResetCaps;

DWORD gIntrTouch        = SYSINTR_TOUCH;
DWORD gIntrTouchChanged = SYSINTR_TOUCH_CHANGED;

static volatile BULVERDE_INTR_REG  *g_pICRegs   = NULL;
static volatile BULVERDE_OST_REG   *g_pOSTRegs  = NULL;
static volatile BULVERDE_GPIO_REG  *g_pGPIORegs = NULL;

//------------------------------------------------------------------------------
// External Functions
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Local Functions
//------------------------------------------------------------------------------

BOOL DdsiTouchPanelEnable(void)
{

    // Allocate touch panel resources (map registers, etc.).
    //
    if (!AllocateTouchPanelResources())
    {
        return(FALSE);
    }

    // Initialize the AC97 link.
    //
    if (!InitializeACLink(FALSE, DEV_TOUCH))
    {
        return(FALSE);
    }

    //HACK: If the capabiliters reads 0x2a0 it's rev 2a, if its 0x2a then its rev 1b .
    ReadAC97(0, (UINT16 *)&g_ResetCaps, DEV_TOUCH);

    // Enable the touch controller.
    //
    DdsiTouchPanelPowerHandler(FALSE);

    return(TRUE);
}


void DdsiTouchPanelDisable(void)
{

    // Clean up any PDD-allocated resources and deinitialize the AC link.
    //
    DeallocateTouchPanelResources();

    DeInitializeACLink(FALSE, DEV_TOUCH);

}


static BOOL AllocateTouchPanelResources(void)
{
    PHYSICAL_ADDRESS RegPA;

    if (g_pICRegs == NULL)
    {
        RegPA.QuadPart = BULVERDE_BASE_REG_PA_INTC;
        g_pICRegs = (volatile BULVERDE_INTR_REG *) MmMapIoSpace(RegPA, 0x400, FALSE);
    }

    if (g_pGPIORegs == NULL)
    {
        RegPA.QuadPart = BULVERDE_BASE_REG_PA_GPIO;
        g_pGPIORegs = (volatile BULVERDE_GPIO_REG *) MmMapIoSpace(RegPA, 0x400, FALSE);
    }

    if (g_pOSTRegs == NULL)
    {
        RegPA.QuadPart = BULVERDE_BASE_REG_PA_OST;
        g_pOSTRegs = (volatile BULVERDE_OST_REG *) MmMapIoSpace(RegPA, 0x400, FALSE);
    }

    if (!g_pOSTRegs  || !g_pGPIORegs || !g_pICRegs)
    {
        DEBUGMSG(1, (TEXT("ERROR: Failed to allocate touch panel resources (Error=%u).\r\n"), GetLastError()));
        DeallocateTouchPanelResources();
        return(FALSE);
    }

    return(TRUE);
}


static void DeallocateTouchPanelResources(void)
{
    if (g_pICRegs)
    {
        VirtualFree((void *)g_pICRegs, 0, MEM_RELEASE);
        g_pICRegs = NULL;
    }
    if (g_pOSTRegs)
    {
        VirtualFree((void *)g_pOSTRegs, 0, MEM_RELEASE);
        g_pOSTRegs = NULL;
    }
    if (g_pGPIORegs)
    {
        VirtualFree((void *)g_pGPIORegs, 0, MEM_RELEASE);
        g_pGPIORegs = NULL;
    }
}


void DdsiTouchPanelPowerHandler(BOOL PowerOff)
{
    UINT16 TCR = 0;

    if (PowerOff)
    {
        // Disable interrupts.
        //
        WriteAC97(UCB_FE_IE, 0, DEV_TOUCH);

        // Unconfigure the AC97 controller.
        UnConfigureAC97Control();
    }
    else
    {
        // Configure the AC97 controller.
        //
        ConfigureAC97Control();
        XllpOstDelayMicroSeconds((P_XLLP_OST_T) g_pOSTRegs, 1000);

        // At power-on, the first interrupt we expect is a pen-down interrupt.
        //
        g_NextExpectedInterrupt = PEN_DOWN;
		{
			INT UncalX, UncalY;
			PDDSampleTouchScreen( &UncalX, &UncalY);
		}
        // Disable the touch interrupt and clear the falling edge
		//
	    WriteAC97(UCB_INT_CS, TSPX_INT, DEV_TOUCH);
		WriteAC97(UCB_FE_IE, 0, DEV_TOUCH);

        // Enable the touch controller interrupt.
        //
        EnableTouchInterrupt();

		// Tell the OAL to clear and unmask the interrupt
		//
		InterruptDone(SYSINTR_TOUCH);
    }
}


BOOL SetUCB1400TouchInterruptMode(void)
{
    UINT16 TCH_CR = 0;

    if (g_ResetCaps == REV_2A)
    {
        TCH_CR = (TSMY_GND | TSPY_GND | TSMX_POW | TSPX_POW);
    }
    else
    {
        TCH_CR = (TSMY_GND | TSPY_GND);
    }

    return(WriteAC97(UCB_TCH_CR, TCH_CR, DEV_TOUCH));
}


LONG DdsiTouchPanelAttach()
{
    return(0);
}


LONG DdsiTouchPanelDetach()
{
    return(0);
}


static void EnableTouchTimerInterrupt(UINT32 TimeInterval)
{
    UINT32 TimerMatch;
    XLLP_OST_HANDLE_T    XllpOSTHandle;

    // Obtain pointers to OST and INTC registers.
    //
    XllpOSTHandle.pOSTRegs  = (XLLP_OST_T *)  g_pOSTRegs;
    XllpOSTHandle.pINTCRegs = (XLLP_INTC_T *) g_pICRegs;

    // XLLI initializes oier and rtsr to zeroes, so no further
    // initialization needs to be done.  Match timers and
    // alarms are disabled.
    //
    // Current usage of Match registers:
    //  M0 - Scheduler
    //  M1 - Touch Panel
    //  M2 - Profiler

    // Configure and arm the timer interrupt  to interrupt every specified system tick interval.
    //
    TimerMatch = (XllpOSTHandle.pOSTRegs->oscr0 + TimeInterval);
    XllpOstConfigureTimer (&XllpOSTHandle, MatchReg1, TimerMatch);

}


static void EnableTouchInterrupt(void)
{

    // Put the UCB1400 into touch interrupt mode.
    //
    SetUCB1400TouchInterruptMode();
    XllpOstDelayMicroSeconds((P_XLLP_OST_T) g_pOSTRegs, 1000);

    // Enable TSPX interrupt (active low).
    //
    WriteAC97(UCB_FE_IE, TSPX_INT, DEV_TOUCH);
}


BOOL PrepareNextInterrupt(void)
{

    if (g_NextExpectedInterrupt == PEN_DOWN)
    {
        // Enable the touch controller interrupt and configure it for a falling edge detection.
        EnableTouchInterrupt();
    }
    else
    {
        // Re/arm the sample timer.
        EnableTouchTimerInterrupt(TOUCH_TIMER_INCREMENT);
    }

    return(TRUE);
}


BOOL PenIsDown(void)
{
    UINT16 TCH_CR = 0;

    if (SetUCB1400TouchInterruptMode())
    {
        XllpOstDelayMicroSeconds((P_XLLP_OST_T) g_pOSTRegs, 250);
        ReadAC97(UCB_TCH_CR, &TCH_CR, DEV_TOUCH);
    }

    return((TCH_CR & TSPX_LOW) ? FALSE : TRUE);
}


void DdsiTouchPanelGetPoint(TOUCH_PANEL_SAMPLE_FLAGS *pTipStateFlags, INT *pUncalX, INT *pUncalY)
{
    static BOOL TouchIrq = TRUE;
    UINT32 InterruptType = SYSINTR_NOP;

    if (!pTipStateFlags || !pUncalX || !pUncalY)
    {
        DEBUGMSG(TRUE, (TEXT("ERROR: DdsiTouchPanelGetPoint: Invalid parameter(s).\r\n")));
        return;
    }

    if (TouchIrq)
    {

        TouchIrq = FALSE;

        // The pen was previously in the up state - it just transitioned to the down state.

        InterruptType   = SYSINTR_TOUCH;
        *pTipStateFlags = PDDSampleTouchScreen(pUncalX, pUncalY);

        // Disable the touch interrupt and clear the falling edge that just occurred.
        WriteAC97(UCB_INT_CS, TSPX_INT, DEV_TOUCH);
        WriteAC97(UCB_FE_IE, 0, DEV_TOUCH);

        // The next expected interrupt will come from the sampling timer (pen-up doesn't
        // cause an interrupt).
        g_NextExpectedInterrupt = PEN_UP_OR_TIMER;

    }
    else
    {

        // The pen was previously in the down state - the sampling timer just interrupted.
        // The pen could now be either up or down at this point - we need to check.

        InterruptType   = SYSINTR_TOUCH_CHANGED;
        *pTipStateFlags = PDDSampleTouchScreen(pUncalX, pUncalY);

        // The next expected interrupt will come from the sampling timer (pen-up doesn't
        // cause an interrupt).
        g_NextExpectedInterrupt = PEN_UP_OR_TIMER;

    }

    // If we're expecting the next interrupt to come from a pen-up or from the timer and
    // the pen isn't currently down, send the MDD a pen-up "event".
    if ((g_NextExpectedInterrupt == PEN_UP_OR_TIMER) && !PenIsDown())
    {

        TouchIrq = TRUE;

        // No TouchSampleDownFlag (so we're telling the MDD that the pen is now up).
        *pTipStateFlags = TouchSampleValidFlag;   // send pen up to mdd

        // The next expected interrupt will come from the pen-down event.
        g_NextExpectedInterrupt = PEN_DOWN;

    }

    // Make sure the next expected interrupt is configured and enabled.
    //
    PrepareNextInterrupt();

    // Tell the OAL to clear and unmask the interrupt the just occurred.
    //
    InterruptDone(InterruptType);

}



// --------------------------------------------


INT evaluateSample(USHORT val0,USHORT val1,USHORT val2,int maxError,INT *sample)
{
    LONG    diff0,diff1,diff2;
    INT   retval=TouchSampleIgnore;

    if ((val0 < MAX_ADC_VAL) && (val1 < MAX_ADC_VAL) && (val2 < MAX_ADC_VAL))
    {
        // Calculate the absolute value of the differences of the sample
        diff0 = val0 - val1;
        diff1 = val1 - val2;
        diff2 = val2 - val0;
        diff0 = diff0 > 0  ? diff0 : -diff0;
        diff1 = diff1 > 0  ? diff1 : -diff1;
        diff2 = diff2 > 0  ? diff2 : -diff2;

        if (diff0 < diff1)
            *sample=(ULONG)(val0 + ((diff2 < diff0) ? val2 : val1));
        else
            *sample=(ULONG)(val2 + ((diff2 < diff1) ? val0 : val1));

        *sample>>=1;

        if ((diff0 < maxError) && (diff1 < maxError) && (diff2 < maxError))
            retval = TouchSampleValidFlag ;
    }
    return(retval);
}


BOOL sampleADC(USHORT *sample,unsigned axis)
{
    UINT32 adccr=0;
    UINT32 adcDataReg;

    BOOL noError=FALSE;

    adccr = (axis == UCB_ADC_X ? ADC_INPUT_TSPY : ADC_INPUT_TSPX) | ADC_ENA ;

    if (WriteAC97(UCB_ADC_CR,*(UINT16 *)&adccr, DEV_TOUCH))
    {
        XllpOstDelayMicroSeconds((P_XLLP_OST_T) g_pOSTRegs, 50);

//        g_pGPIORegs->GPCR0 = XLLP_GPIO_BIT_PWM_OUT0;

        adccr |= ADC_START;
        if (WriteAC97(UCB_ADC_CR,*(UINT16 *)&adccr, DEV_TOUCH))
        {
//            g_pGPIORegs->GPSR0 = XLLP_GPIO_BIT_PWM_OUT0;

            XllpOstDelayMicroSeconds((P_XLLP_OST_T) g_pOSTRegs, 50);

            do
            {                     // wait for sample completion
                ReadAC97(UCB_ADC_DATA,(UINT16 *) &adcDataReg,DEV_TOUCH);

            } while (!TEST(adcDataReg,ADC_DATA_VAL));



            *sample=TEST(adcDataReg,ADC_DATA) ;
            // disable adc
            CLEAR(adccr,ADC_ENA);
            WriteAC97(UCB_ADC_CR,*(UINT16 *)&adccr,DEV_TOUCH);
            noError=TRUE;
        }
    }

    return(noError);
}


USHORT getTouchCoordinate(unsigned axis)
{
    UINT32 tcr=0;
    BOOL noError=FALSE;
    USHORT sample=MAX_ADC_VAL;

    //Bias the ADC, set position mode and choose the correct axis.

    tcr = TSC_BIAS_ENA | TSC_MODE_POSITION | ((axis==UCB_ADC_X) ? (TSPX_POW | TSMX_GND) : (TSPY_POW | TSMY_GND));
    if (WriteAC97(UCB_TCH_CR,*(UINT16 *)&tcr, DEV_TOUCH))
        noError=sampleADC(&sample,axis);

    return(sample);
}


extern "C" BOOL TouchDriverCalibrationPointGet(TPDC_CALIBRATION_POINT *pTCP)
{
    INT32 cDisplayWidth = pTCP->cDisplayWidth;
    INT32 cDisplayHeight = pTCP->cDisplayHeight;

    int CalibrationRadiusX = cDisplayWidth/10;
    int CalibrationRadiusY = cDisplayHeight/10;


    switch (pTCP->PointNumber)
    {
    case  0:  // Middle
        pTCP->CalibrationX = cDisplayWidth/2;
        pTCP->CalibrationY = cDisplayHeight/2;
        break;

    case  1:  // Upper Left
        pTCP->CalibrationX = CalibrationRadiusX*2;
        pTCP->CalibrationY = CalibrationRadiusY*2;
        break;

    case  2:  // Lower Left
        pTCP->CalibrationX = CalibrationRadiusX*2;
        pTCP->CalibrationY = cDisplayHeight - CalibrationRadiusY*2;
        break;

    case  3:  // Lower Right
        pTCP->CalibrationX = cDisplayWidth - CalibrationRadiusX*2;
        pTCP->CalibrationY = cDisplayHeight - CalibrationRadiusY*2;
        break;

    case  4:  // Upper Right
        pTCP->CalibrationX = cDisplayWidth - CalibrationRadiusX*2;
        pTCP->CalibrationY = CalibrationRadiusY*2;
        break;

    default:
        pTCP->CalibrationX = cDisplayWidth/2;
        pTCP->CalibrationY = cDisplayHeight/2;
        SetLastError(ERROR_INVALID_PARAMETER);
        return (FALSE);
    }

    return (TRUE);
}


extern "C" BOOL DdsiTouchPanelGetDeviceCaps(INT iIndex,LPVOID  lpOutput)
{

    if (lpOutput == NULL)
    {
        ERRORMSG(1,(__TEXT("TouchPanelGetDeviceCaps: invalid parameter.\r\n")));
        SetLastError(ERROR_INVALID_PARAMETER);
        return (FALSE);
    }

    switch (iIndex)
    {
    case TPDC_SAMPLE_RATE_ID:
        {
            TPDC_SAMPLE_RATE  *pTSR = (TPDC_SAMPLE_RATE*)lpOutput;

            pTSR->SamplesPerSecondLow = TOUCHPANEL_SAMPLE_RATE_LOW;
            pTSR->SamplesPerSecondHigh = TOUCHPANEL_SAMPLE_RATE_HIGH;
            pTSR->CurrentSampleRateSetting = 90;
        }
        break;

    case TPDC_CALIBRATION_POINT_COUNT_ID:
        {
            TPDC_CALIBRATION_POINT_COUNT *pTCPC = (TPDC_CALIBRATION_POINT_COUNT*)lpOutput;
            pTCPC->flags = 0;
            pTCPC->cCalibrationPoints = 5;
        }
        break;

    case TPDC_CALIBRATION_POINT_ID:
        return(TouchDriverCalibrationPointGet((TPDC_CALIBRATION_POINT*)lpOutput));

    default:
        ERRORMSG(1,(__TEXT("TouchPanelGetDeviceCaps: invalid parameter.\r\n")));
        SetLastError(ERROR_INVALID_PARAMETER);
        return (FALSE);
    }
    return (TRUE);
}


BOOL DdsiTouchPanelSetMode(INT iIndex,LPVOID  lpInput)
{
    BOOL  ReturnCode = FALSE;

    switch (iIndex)
    {
    case TPSM_SAMPLERATE_LOW_ID:
    case TPSM_SAMPLERATE_HIGH_ID:
        SetLastError(ERROR_SUCCESS);
        ReturnCode = TRUE;
        break;

    default:
        SetLastError(ERROR_INVALID_PARAMETER);
        break;
    }
    return(ReturnCode);
}


TOUCH_PANEL_SAMPLE_FLAGS PDDSampleTouchScreen(INT *x,INT *y)
{
    UINT32 i;
    TOUCH_PANEL_SAMPLE_FLAGS TmpStateFlags;
    TOUCHPANEL_POINT_SAMPLES rgPointSamples;
    INT TmpX = 0;
    INT TmpY = 0;

    // It is more reliable to read X and Y alternatively rather then read all X's then all Y's.
    // This way we have a better chance to detect and reject a bad sample if the pen is up
    // briefly while the reading takes place. Otherwise, we might have all three X readings
    // when pen is up momentarily and all three Y readings when pen is down again. The
    // function evaluateSample() will not be able to detect and reject the sample in this case.
    //
    for(i=0;i<NUMBER_SAMPLES_PER_POINT;i++)
    {
        //getTouchCoordinate(UCB_ADC_X);             // Prime the Pump for the X
        rgPointSamples[i].XSample=getTouchCoordinate(UCB_ADC_X);

        //getTouchCoordinate(UCB_ADC_Y);             // Prime the Pump for the Y
        rgPointSamples[i].YSample=getTouchCoordinate(UCB_ADC_Y);
    }

    TmpStateFlags = TouchSampleDownFlag;
    if (evaluateSample(rgPointSamples[0].XSample,rgPointSamples[1].XSample,rgPointSamples[2].XSample,
                       DELTA_X_COORD_VARIANCE,&TmpX) == TouchSampleIgnore)
    {
        TmpStateFlags |= TouchSampleIgnore;
//        DEBUGMSG(1,(TEXT("Invalid X sample\r\n")));
    }
    else
    {
        TmpStateFlags |= evaluateSample(rgPointSamples[0].YSample,rgPointSamples[1].YSample,rgPointSamples[2].YSample,
                                        DELTA_Y_COORD_VARIANCE,&TmpY);
    }

    *x=TmpX;
    *y=TmpY;

    return(TmpStateFlags);
}

