;
; Copyright (c) Microsoft Corporation.  All rights reserved.
;
;
; Use of this sample source code is subject to the terms of the Microsoft
; license agreement under which you licensed this sample source code. If
; you did not accept the terms of the license agreement, you are not
; authorized to use this sample source code. For the terms of the license,
; please see the license agreement between you and Microsoft or, if applicable,
; see the LICENSE.RTF on your install media or the root of your tools installation.
; THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
;
;------------------------------------------------------------------------------
;
;  File:  image_cfg.inc
;
;  Defines configuration parameters used to create the NK and Bootloader
;  program images.
;

    IF  !:DEF: image_cfg_inc
image_cfg_inc    EQU 1


;;------------------------------------------------------------------------------
;;  RESTRICTION
;;
;;  This file is a configuration file. It should ONLY contain simple #define 
;;  directives defining constants. This file is included by other files that 
;;  only support simple substitutions.
;;
;;------------------------------------------------------------------------------

;;------------------------------------------------------------------------------
;;  NAMING CONVENTION
;;
;;  The IMAGE_ naming convention ...
;;
;;  IMAGE_<NAME>_<SECTION>_<MEMORY_DEVICE>_[OFFSET|SIZE|START|END]
;;
;;      <NAME>          - WINCE, BOOT, SHARE
;;      <SECTION>       - section name: user defined
;;      <MEMORY_DEVICE> - the memory device the block resides on
;;      OFFSET          - number of bytes from memory device start address
;;      SIZE            - maximum size of the block
;;      START           - start address of block    (device address + offset)
;;      END             - end address of block      (start address  + size - 1)
;;
;;------------------------------------------------------------------------------

IMAGE_SHARE_ARGS_UA_START           EQU    0xA00FF000
IMAGE_SHARE_ARGS_CA_START           EQU    0x800FF000
IMAGE_SHARE_ARGS_SIZE               EQU    0x00001000

IMAGE_BOOT_BLDRIMAGE_FLASH_OFFSET   EQU    0x00000000
IMAGE_BOOT_BLDRIMAGE_RAM_OFFSET     EQU    0x00030000
IMAGE_BOOT_BLDRIMAGE_RAM_SIZE       EQU    0x00040000

IMAGE_BOOT_BLDRIMAGE_FLASH_PA_START EQU    (MAINSTONEII_BASE_PA_BOOT_FLASH + IMAGE_BOOT_BLDRIMAGE_FLASH_OFFSET)
IMAGE_BOOT_BLDRIMAGE_FLASH_PA_END   EQU    (IMAGE_BOOT_BLDRIMAGE_FLASH_PA_START + IMAGE_BOOT_BLDRIMAGE_RAM_SIZE - 1)
IMAGE_BOOT_BLDRIMAGE_RAM_PA_START   EQU    (MAINSTONEII_BASE_PA_SDRAM + IMAGE_BOOT_BLDRIMAGE_RAM_OFFSET)
IMAGE_BOOT_BLDRIMAGE_RAM_PA_END     EQU    (IMAGE_BOOT_BLDRIMAGE_RAM_PA_START + IMAGE_BOOT_BLDRIMAGE_RAM_SIZE - 1)

IMAGE_BOOT_STACK_RAM_PA_START       EQU    0xA0010000
IMAGE_BOOT_STACK_RAM_UA_START       EQU    0xA0010000
IMAGE_BOOT_STACK_RAM_CA_START       EQU    0x80010000

IMAGE_BOOT_CONFIG_FLASH_OFFSET      EQU    0x00040000

IMAGE_BOOT_IPL_FLASH_OFFSET         EQU    0x00080000
IMAGE_BOOT_IPL_FLASH_MAXSIZE        EQU    0x00040000
IMAGE_BOOT_IPL_FLASH_PA_START       EQU    (MAINSTONEII_BASE_PA_BOOT_FLASH + IMAGE_BOOT_IPL_FLASH_OFFSET)
IMAGE_BOOT_IPL_FLASH_PA_END         EQU    (IMAGE_BOOT_IPL_FLASH_PA_START + IMAGE_BOOT_IPL_FLASH_MAXSIZE - 1)

; match IPL.BIB
IMAGE_BOOT_IPL_RAM_OFFSET           EQU    0x00020000
IMAGE_BOOT_IPL_RAM_SIZE             EQU    0x00020000
IMAGE_BOOT_IPL_RAM_PA_START         EQU    (MAINSTONEII_BASE_PA_SDRAM + IMAGE_BOOT_IPL_RAM_OFFSET)
IMAGE_BOOT_IPL_RAM_PA_END           EQU    (IMAGE_BOOT_IPL_RAM_PA_START + IMAGE_BOOT_IPL_RAM_SIZE - 1)

IMAGE_OSIMAGE_FLASH_OFFSET          EQU    0x000C0000
IMAGE_OSIMAGE_FLASH_MAXSIZE         EQU    (MAINSTONEII_SIZE_BOOT_FLASH - IMAGE_OSIMAGE_FLASH_OFFSET)

;IMAGE_BOOT_DOWNLOADCACHE_RAM_OFFSET EQU    0x00060000
;IMAGE_BOOT_DOWNLOADCACHE_RAM_SIZE   EQU    0x00080000

;;------------------------------------------------------------------------------

    ENDIF
    END
