;
; Copyright (c) Microsoft Corporation.  All rights reserved.
;
;
; Use of this source code is subject to the terms of the Microsoft end-user
; license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
; If you did not accept the terms of the EULA, you are not authorized to use
; this source code. For a copy of the EULA, please see the LICENSE.RTF on your
; install media.
;
;++
;
; Module Name:
;
;    mainstoneii.inc
;
; Abstract:
;
;    Mainstone II board definitions.
;
; Environment:
;
; Revision History:
;
;--
;
; Copyright  2002-2003 Intel Corp.
;
;-------------------------------------------------------------------------------
;

    IF  !:DEF: mainstoneii_inc_
mainstoneii_inc_  EQU 1

    INCLUDE gpio.inc
;    INCLUDE mainstoneii_macros.inc

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;       
;;;;;   MAINSTONE CHIP SELECTS
;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;
; MainstoneII: (on PDC): nCS0: Boot FLASH (32MB - 64MB on the MCPII module)
;
MAINSTONEII_BASE_PA_BOOT_FLASH           EQU     0x00000000
;  IF :DEF: MCPII_MODULE
MAINSTONEII_SIZE_BOOT_FLASH              EQU     0x04000000
;  ELSE
;MAINSTONEII_SIZE_BOOT_FLASH              EQU     0x02000000
;  ENDIF
    
;
; MainstoneII: nCS1: Secondary FLASH (32 MB)
;
;MAINSTONEII_BASE_PA_SECONDARY_FLASH      EQU     0x04000000
    
;
; MainstoneII: nCS1: Board-Level Registers (FPGA)
;
;MAINSTONEII_BASE_REG_PA_FPGA             EQU     0x04000000
    
;
; MainstoneII: nCS2 (on PDC): SRAM (2 MB)
;
MAINSTONEII_BASE__PA_SRAM                EQU     0x0A000000

;
; MainstoneII: nCS3:  not used, not mapped.  The GPIO is used as PCMCIA PSKTSEK signal on MainstoneII.
;
MAINSTONEII_BASE_PA_CS3                  EQU     0x0C000000

;
; MainstoneII: nCS2: Ethernet (DM9000)
;
MAINSTONEII_BASE_REG_PA_DM9000_ETHERNET    EQU     0x08000000

;
; MainstoneII: nCS5: eXpansion Board Header
;
MAINSTONEII_BASE_PA_XDC                  EQU     0x14000000

;
; MainstoneII: SDRAM base address
;
MAINSTONEII_BASE_PA_SDRAM                EQU     0xA0000000
  IF :DEF: SDRAM_SIZE_64_MB
MAINSTONEII_SIZE_SDRAM                   EQU     0x04000000
  ELSE
MAINSTONEII_SIZE_SDRAM                   EQU     0x08000000
  ENDIF

  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;       
;;;;;   MAINSTONE BASEBOARD REGISTER OFFSETS
;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;
; MainstoneII Platform Registers' offsets from FPGA_REGS_BASE_... (FPGA on lower half of nCS2)
;
LEDDAT1_OFFSET                  EQU     0x00000010          ; Hex LED Data Register 1. (8 LEDS DATA U19 (MSN),21,23,25,18,20,22,24 (LSN)) 
LEDDAT2_OFFSET                  EQU     0x00000014          ; Hex LED Data Register 2. (Decimal points and dots)
LEDCTL_OFFSET                   EQU     0x00000040          ; LED Control (Blanking for HEX LEDs, discrete LED on/off)
GPSW_OFFSET                     EQU     0x00000060          ; General Purpose Switches + HEX Rotaries Status (read-only)        
MISCWR1_OFFSET                  EQU     0x00000080          ; Miscellaneous Write Register 1
MISCWR2_OFFSET                  EQU     0x00000084          ; Miscellaneous Write Register 2
MISCRD_OFFSET                   EQU     0x00000090          ; Miscellaneous Read Register
INTMSKEN1_OFFSET                EQU     0x000000C0          ; Platform Interrupt Mask/Enable 1
INTSETCLR_OFFSET                EQU     0x000000D0          ; Platform Interrupt Set/Clear 1
PCMCIA0SRCR_OFFSET              EQU     0x000000E0          ; PCMCIA Socket 0 Status/Control
PCMCIA1SRCR_OFFSET              EQU     0x000000E4          ; PCMCIA Socket 1 Status/Control


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;       
;;;;;   MAINSTONEII Memory Controller Values
;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; SDRAM Settings
;
MDCNFG_VAL     EQU    0x00000AC9    ; SDRAM Config Reg (CL=3, Normal Addressing, 4 Banks, 13 row bits, 9 col bits, 32-bit port width)

MDREFR_VAL     EQU    0x2093A01E    ; SDRAM Refresh Reg (SDCLK[0]=MemClk/4, APD, SDCLK[1]=MemClk/2, DRI=x01E (7.62us refreshes))

MDMRS_VAL      EQU    0x00000000    ; SDRAM Mode Reg Set Config Reg (MDMRS1: burst reads & writes)
MDMRSLP_VAL    EQU    0x0000C008    ; SDRAM Mode Reg Set Config Reg - Low Power (enabled, 45 C TCR, all banks PASR (partial array self refresh))

; Static Memory Settings
;
K3_MSC0_VAL    EQU    0x39F2A7A3    ; Static Mem. Control Reg 0 (nCS0,1)=(Boot Flash, Secondary Flash)
J3_MSC0_VAL    EQU    0xA7A339F2    ; just reversed K3's

L3_MSC0_VAL    EQU    0xA7A339F2    ; just reversed K3's - Borrowing J3's for ASync. Tyax at the moment


MSC1_VAL       EQU    0x0000A691    ; Static Mem. Control Reg 1 (nCS2,3)=(FPGA/SRAM, unused)

MSC2_VAL       EQU    0x0000B884    ; Static Mem. Control Reg 2 (nCS4,5)=(SMSC, expansion header)

; PCMCIA and CF Interfaces
;
MECR_VAL       EQU    0x00000001    ; Expansion memory (PCMCIA/CF) Bus Config (NOS=2, card is NOT there)
MCMEM0_VAL     EQU    0x0001C391    ; Card I-face Common Mem Space socket 0 timing config
MCMEM1_VAL     EQU    0x0001C391    ; Card I-face Common Mem Space socket 1 timing config
MCATT0_VAL     EQU    0x0001C391    ; Card I-face Attribute Space socket 0 timing config 
MCATT1_VAL     EQU    0x0001C391    ; Card I-face Attribute Space socket 1 timing config 
MCIO0_VAL      EQU    0x0001C391    ; Card I-face I/O Space socket 0 timing config       
MCIO1_VAL      EQU    0x0001C391    ; Card I-face I/O Space socket 1 timing config       

; Synch. Static Memory: Processor Daughter Card's K18 FLASH is capable of operating synchronously.
;
SXCNFG_VAL     EQU    0x4000600D    ; Synch. Static Mem. Config. Reg. (Burst-of-8 reads, CL=4, SDCLK return latching)

; Alternate Bus Masters - Fly-By-DMA Mode (unused on MainstoneII)
;
FLYCNFG_VAL    EQU    0x00010001    ; Fly-by-DMA config. reg (==POR: both DVALs active low)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;       
;;;;;   MAINSTONEII GPIO values
;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; GPIO Direction
;
GPDR0_VAL               EQU     (0x00709C04:OR:0xC0000000)
GPDR1_VAL               EQU     0x00CF0382    ; plus ffuart, PCMCIA and MBREQ/GNT
GPDR2_VAL               EQU     0x0021C000    ; plus pcmcia stuff
GPDR3_VAL               EQU     0x00020002    ; MBGNT (output) enabled + AC reset output

; GPIO Rising-Edge Detect
;
GRER0_VAL               EQU     0x00000000    
GRER1_VAL               EQU     0x00000000
GRER2_VAL               EQU     0x00000000
GRER3_VAL               EQU     0x00000000

; GPIO Falling-Edge Detect: gpio[1:0]
;
GFER0_VAL               EQU     0x00000000    
GFER1_VAL               EQU     0x00000000
GFER2_VAL               EQU     0x00000000
GFER3_VAL               EQU     0x00000000

; GPIO Alternate Functions
;                                              
GAFR0_L_VAL             EQU     (GPIO_11_AF1_EXTSYNC0_CHOUT0:OR:GPIO_13_AF1_CLKEXT:OR:GPIO_15_AF2_nCS1)
GAFR0_U_VAL             EQU     (GPIO_18_AF1_RDY:OR:GPIO_20_AF1_DREQ0_nSDCS2:OR:GPIO_21_AF1_nSDCS3:OR:GPIO_28_AF1_AC97_I2S_BITCLK:OR:GPIO_29_AF1_AC97SDATAIN0:OR:GPIO_30_AF2_AC97SDATAOUT:OR:GPIO_31_AF2_AC97SYNC)
GAFR1_L_VAL             EQU     (GPIO_33_AF2_nCS5:OR:GPIO_34_AF1_FFRXD:OR:GPIO_35_AF1_FFCTS:OR:GPIO_36_AF1_FFDCD:OR:GPIO_37_AF1_FFDSR:OR:GPIO_38_AF1_FFRI:OR:GPIO_39_AF2_FFTXD:OR:GPIO_40_AF2_FFDTR:OR:GPIO_41_AF2_FFRTS)
GAFR1_U_VAL             EQU     (GPIO_48_AF2_PCMCIAnPOE:OR:GPIO_49_AF2_nPWE:OR:GPIO_50_AF2_PCMCIAnPIOR:OR:GPIO_51_AF2_PCMCIAnPIOW:OR:GPIO_54_AF2_BBOBWAIT_PCMCIAnPCE2:OR:GPIO_55_AF2_BBIBDAT1_PCMCIAnPREG:OR:GPIO_56_AF1_PCMCIAnPWAIT:OR:GPIO_57_AF1_PCMCIAnIOIS16)
GAFR2_L_VAL             EQU     (GPIO_78_AF2_nCS2:OR:GPIO_79_AF1_PCMCIAPSKTSEL)
GAFR2_U_VAL             EQU     (GPIO_80_AF2_MBREQ_nCS4:OR:GPIO_85_AF1_PCMCIAnPCE1)
GAFR3_L_VAL             EQU     (0x00000000) 
GAFR3_U_VAL             EQU     (0x00000000)

; GPIO Set Regs
;
GPSR0_VAL               EQU     0x00709C04
GPSR1_VAL               EQU     0x00CF0002    ; plus pcmcia
GPSR2_VAL               EQU     0x0021C000    ; plus pcmcia + mbgnt
GPSR3_VAL               EQU     0x00020000    ; Start with AC97 reset  inactive (high)

; GPIO Clear Regs
;
GPCR0_VAL               EQU     0xC0040000    ; AC sync and data out low
GPCR1_VAL               EQU     0x00000380    ; ffuart
GPCR2_VAL               EQU     0x00000000
GPCR3_VAL               EQU     0x00000003


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;       
;;;;;   MAINSTONEII BASEBOARD REGISTER (FPGA) values
;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Hex LED Initial Settings
;
HEXLED1_VAL             EQU     0x00000000    ; Zero-out LEDs
HEXLED2_VAL             EQU     0x0000FF3F    ; Turn off decimals and other dots on the HEX LEDs.
LEDCTL_ALL              EQU     0x000000FF    ; HEX LEDs ON, all discretes off.

; MISC_WR Initial Settings
;
MISCWR1_VAL             EQU     0x00000020    ; everything off, PCMCIA/BB Mux: route signals to PCMCIA.
MISCWR2_VAL             EQU     0x00000010    ; everything off including USB client detection

; INT register Initial Settings
;
INTMSKEN1_VAL           EQU     0x00000000    ; mask all ints
INTSETCLR1_VAL          EQU     0x00000000    ; clear all ints


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;       
;;;;;   MAINSTONEII VIRTUAL CONFIG REGISTERS (GPIOx) values
;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; SCR Masks & Constants.  *The SCR is not a register: it is the result of reading GPIO[73:58] before assigning an alternate function to them.
;   Therefore, these constants can be used once this value is is read, masked correctly and stored off to driver globals.
;
SCR_SWAP_FLASH_MASK     EQU     (0x1 << 15)
SCR_EXBID_MASK          EQU     (0x3 << 13)
SCR_nEXB_PRES_MASK      EQU     (0x1 << 12)
SCR_MBREV_MASK          EQU     (0x7 << 9)
SCR_nMBPRES_MASK        EQU     (0x1 << 8)
SCR_DCREV_MASK          EQU     (0x7 << 5)
SCR_DCID_MASK           EQU     (0xF)

SCR_MBREV_1p0           EQU     0x0           ; Main Board Revision 1.0
SCR_MBREV_1p1           EQU     0x1           ; Main Board Revision 1.1
SCR_DCREV_1p0           EQU     0x0           ; Daughter Card Revision 1.0
SCR_DCREV_1p1           EQU     0x1           ; Daughter Card Revision 1.1

; LCDCR Masks & Constants *The SCR is not a register: it is the result of reading GPIO[87,86,77,75,74,19,14] before assigning an alternate function to them.
;
LCDCR_ORIENT_MASK       EQU     (0x1 << 6)
LCDCR_LCDID_MASK        EQU     (0x3F)

LCDCR_ORIENT_LANDSCAPE               EQU     0x1
LCDCR_LCDID_TOSHIBA_LTM04C380K_V18   EQU     0X1    ; 256 KB
LCDCR_LCDID_TOSHIBA_LTM04C380K_V16   EQU     0      ; 64 KB


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;       
;;;;;   MAINSTONEII BASEBOARD REGISTER MASKS
;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; GP Switches
;
SW12_MASK               EQU     (0x1 << 15)
SW11_MASK               EQU     (0x1 << 14)
SW10_MASK               EQU     (0x1 << 13)
SW8_MASK                EQU     (0x1 << 12)
SW7_MASK                EQU     (0x1 << 11)
SW6_MASK                EQU     (0x1 << 10)
SW4_MASK                EQU     (0x1 << 9)
SW3_MASK                EQU     (0x1 << 8)
HEXSW5_MASK             EQU     (0xF << 4)
HEXSW9_MASK             EQU     (0xF)


; -----------------------------------------------------------------------------
; -----------------------------------------------------------------------------
;
; Processor stepping Values
;

BULVERDE_CP15_A0_VAL  EQU (0x69054110)
BULVERDE_CP15_A1_VAL  EQU (0x69054111)
BULVERDE_CP15_B0_VAL  EQU (0x69054112)

BULVERDE_JTAG_A0_VAL  EQU (0x09265013)
BULVERDE_JTAG_A1_VAL  EQU (0x19265013)
BULVERDE_JTAG_B0_VAL  EQU (0x29265013)

A0_STEPPING           EQU (0x0)
A1_STEPPING           EQU (0x1)
B0_STEPPING           EQU (0x2)


TOUCH_GLOBALS_OFFSET        EQU (0x0)
PCMCIA_GLOBALS_OFFSET       EQU (0x100)
PROFILE_GLOBALS_OFFSET      EQU (0x200)
MISC_GLOBALS_OFFSET         EQU (0x300)
PMU_GLOBALS_OFFSET          EQU (0x400)
AC97_GLOBALS_OFFSET         EQU (0x500)
UNINIT_MISC_GLOBALS_OFFSET  EQU (0x800)
DBG_ETH_GLOBALS_OFFSET      EQU (0x900)

  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Sleep Data area definitions
;

; Must keep in synch with MEMDefs.h and config.bib

    IF :DEF: MMXIP_MEMMAP
SLEEP_SAVE_OFFSET           EQU 0x2BA000
    ELSE
SLEEP_SAVE_OFFSET           EQU 0xA000
    ENDIF ;MMXIP_MEMMAP

SLEEP_SAVE_BYTES            EQU 0x1000      ; total size of sleep save area
SLEEP_SAVE_PHYSICAL_BASE    EQU (MAINSTONEII_BASE_PA_SDRAM + SLEEP_SAVE_OFFSET)

; Working backward from end.  All 4-byte data elements.
SLEEP_TYPE_OFFSET               EQU (SLEEP_SAVE_BYTES-4)
RESET_LAUNCH_ADDR_OFFSET        EQU (SLEEP_TYPE_OFFSET-4)        
RESET_OEM_ADDR_TABLE_PA_OFFSET  EQU (RESET_LAUNCH_ADDR_OFFSET-4)
RESET_KERNEL_START_PA_OFFSET    EQU (RESET_OEM_ADDR_TABLE_PA_OFFSET-4)

RESET_LAUNCH_ADDR_PHYSICAL      EQU  (SLEEP_SAVE_PHYSICAL_BASE + RESET_LAUNCH_ADDR_OFFSET)
SLEEP_TYPE_PHYSICAL             EQU  (SLEEP_SAVE_PHYSICAL_BASE + SLEEP_TYPE_OFFSET)

; Define SLEEP TYPES and related constants here

    ; Normal sleep with standard resume desired is standard
    ; Use 0 as basic mode for simplicity, because this area is zero-initialized
    ;   at power up.
SLEEP_TYPE_STANDARD         EQU  0  
SLEEP_TYPE_SOFT_RESET       EQU  1  
;SLEEP_TYPE_ETHDBG_PART_INIT EQU  2

;
;  End Sleep Data area definitions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ; //////////////////////////////////////////////////////////////////////    
    ; This macro compares the CPU ID located in Driver Globals against a given
    ; CPU ID Value

    MACRO
    IsA0CPUID $w1, $w2
        ldr $w1, =DRIVER_GLOBALS_CPU_ID_LOCATION
        ldr $w2, [$w1]    ; Start of driver globals misc section
        ldr $w1, =BULVERDE_CP15_A0_VAL
        cmp $w1, $w2
    MEND

    MACRO
    IsA1CPUID $w1, $w2
        ldr $w1, =DRIVER_GLOBALS_CPU_ID_LOCATION
        ldr $w2, [$w1]    ; Start of driver globals misc section
        ldr $w1, =BULVERDE_CP15_A1_VAL
        cmp $w1, $w2
    MEND

    MACRO
    IsB0CPUID $w1, $w2
        ldr $w1, =DRIVER_GLOBALS_CPU_ID_LOCATION
        ldr $w2, [$w1]    ; Start of driver globals misc section
        ldr $w1, =BULVERDE_CP15_B0_VAL
        cmp $w1, $w2
    MEND


    ENDIF    ; mainstoneii_inc_.
    
    END
