;
; Copyright (c) Microsoft Corporation.  All rights reserved.
;
;
; Use of this source code is subject to the terms of the Microsoft end-user
; license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
; If you did not accept the terms of the EULA, you are not authorized to use
; this source code. For a copy of the EULA, please see the LICENSE.RTF on your
; install media.
;
;++
;
; Module Name:
;
;    oemaddrtab_cfg.inc
;
; Abstract:
;
;  This file is used to define g_oalAddressTable. This table is passed to
;  KernelStart to estabilish physical to virtual memory mapping. This table
;  is used also in IOMEM OAL module to map between physical and virtual
;  memory addresses via OALPAtoVA/OALVAtoPA functions.
;
; Environment:
;
; Revision History:
;
;--
;
; Copyright  2002-2003 Intel Corp.
;
;-------------------------------------------------------------------------------
;

; Export Definition

        EXPORT  g_oalAddressTable[DATA]

;------------------------------------------------------------------------------
;
; TABLE FORMAT
;       cached address, physical address, size
;------------------------------------------------------------------------------
; Note:  Items on mainboard not supported by BSP memory map as it puts total over 512MB:
;       nCS1 : 32MB of Secondary Flash
;

ALIGN
g_oalAddressTable

;C5 processor card added 64MB of additional RAM. Address table was changed to support the additional
;RAM and to free up some extra space by reducing allocation to PCMCIA.
;        DCD     0x80000000, 0xA0000000, 64      ; MAINSTONEII: SDRAM (64MB).
;        DCD     0x84000000, 0x5C000000,  1      ; BULVERDE: Internal SRAM (64KB bank 0).
;        DCD     0x84100000, 0x58000000,  1      ; BULVERDE: Internal memory PM registers.
;        DCD     0x84200000, 0x4C000000,  1      ; BULVERDE: USB host controller.
;        DCD     0x84300000, 0x48000000,  1      ; BULVERDE: Memory controller.
;        DCD     0x84400000, 0x44000000,  1      ; BULVERDE: LCD controller.
;        DCD     0x84500000, 0x40000000, 32      ; BULVERDE: Memory-mapped registers (peripherals).
;        DCD     0x86500000, 0x3C000000, 64      ; BULVERDE: PCMCIA S1 common memory space.
;        DCD     0x8A500000, 0x38000000, 64      ; BULVERDE: PCMCIA S1 attribute memory space.
;        DCD     0x8E500000, 0x30000000, 32      ; BULVERDE: PCMCIA S1 I/O space.
;        DCD     0x90500000, 0x2C000000, 64      ; BULVERDE: PCMCIA S0 common memory space.
;        DCD     0x94500000, 0x28000000, 64      ; BULVERDE: PCMCIA S0 attribute memory space.
;        DCD     0x98500000, 0x20000000, 32      ; BULVERDE: PCMCIA S0 I/O space.

  IF :DEF: SDRAM_SIZE_64_MB
        DCD     0x80000000, 0xA0000000, 64      ; MAINSTONEII: SDRAM (64MB).
  ELSE
        DCD     0x80000000, 0xA0000000, 128     ; MAINSTONEII: SDRAM (128MB).
  ENDIF
        DCD     0x88000000, 0x5C000000,   1     ; BULVERDE: Internal SRAM (64KB bank 0).
        DCD     0x88100000, 0x58000000,   1     ; BULVERDE: Internal memory PM registers.
        DCD     0x88200000, 0x4C000000,   1     ; BULVERDE: USB host controller.
        DCD     0x88300000, 0x48000000,   1     ; BULVERDE: Memory controller.
        DCD     0x88400000, 0x44000000,   1     ; BULVERDE: LCD controller.
        DCD     0x88500000, 0x40000000,  32     ; BULVERDE: Memory-mapped registers (peripherals).
        DCD     0x8A500000, 0x3C000000,  16     ; BULVERDE: PCMCIA S1 common memory space.
        DCD     0x8B500000, 0x38000000,  64     ; BULVERDE: PCMCIA S1 attribute memory space.
        DCD     0x8F500000, 0x30000000,   1     ; BULVERDE: PCMCIA S1 I/O space.
        DCD     0x8F600000, 0x2C000000,  16     ; BULVERDE: PCMCIA S0 common memory space.
        DCD     0x90600000, 0x28000000,  64     ; BULVERDE: PCMCIA S0 attribute memory space.
        DCD     0x94600000, 0x20000000,   1     ; BULVERDE: PCMCIA S0 I/O space.
        DCD     0x9A500000, 0xE0000000,  1      ; MAINSTONEII: Zero-bank (in reserved slot - no physical memory required).
        DCD     0x9A600000, 0x14000000,  1      ; MAINSTONEII: nCS5: eXpansion board header.
        DCD     0x9A700000, 0x10000000,  1      ; MAINSTONEII: nCS4: SMSC 91C111 Ethernet controller.
        DCD     0x9A800000, 0x0A000000,  1      ; MAINSTONEII: nCS2 (upper half): 2MB SRAM.
        DCD     0x9A900000, 0x08000000,  1      ; MAINSTONEII: nCS2 (lower half): Board registers (FPGA).
;  IF :DEF: MCPII_MODULE
        DCD     0x9AA00000, 0x00000000, 64      ; MAINSTONEII: nCS0: MCPII Module Boot Flash (64MB).
;  ELSE 
;        DCD     0x9AA00000, 0x04000000, 32      ; MAINSTONEII: nCS1: Secondary flash (32MB).
;        DCD     0x9CA00000, 0x00000000, 32      ; MAINSTONEII: nCS0: Boot Flash (32MB).
;  ENDIF
        DCD     0x9EA00000, 0x50000000,  1      ; BULVERDE: Camera peripheral interface.
        
        DCD     0x00000000, 0x00000000,  0      ; end of table

;------------------------------------------------------------------------------

        END
